@@ -26,12 +26,16 @@ import org.apache.lucene.codecs.CodecUtil;
 import org.apache.lucene.codecs.DocValuesProducer;
 import org.apache.lucene.index.BinaryDocValues;
 import org.apache.lucene.index.CorruptIndexException;
+import org.apache.lucene.index.DocValues;
 import org.apache.lucene.index.FieldInfo;
 import org.apache.lucene.index.IndexFileNames;
 import org.apache.lucene.index.NumericDocValues;
+import org.apache.lucene.index.RandomAccessOrds;
 import org.apache.lucene.index.SegmentReadState;
 import org.apache.lucene.index.SortedDocValues;
+import org.apache.lucene.index.SortedNumericDocValues;
 import org.apache.lucene.index.SortedSetDocValues;
+import org.apache.lucene.store.ChecksumIndexInput;
 import org.apache.lucene.store.IndexInput;
 import org.apache.lucene.util.Bits;
 import org.apache.lucene.util.BytesRef;
@@ -45,48 +49,55 @@ import org.apache.lucene.util.RamUsageEstimator;
 
 class DirectDocValuesProducer extends DocValuesProducer {
   // metadata maps (just file pointers and minimal stuff)
-  private final Map<Integer,NumericEntry> numerics = new HashMap<Integer,NumericEntry>();
-  private final Map<Integer,BinaryEntry> binaries = new HashMap<Integer,BinaryEntry>();
-  private final Map<Integer,SortedEntry> sorteds = new HashMap<Integer,SortedEntry>();
-  private final Map<Integer,SortedSetEntry> sortedSets = new HashMap<Integer,SortedSetEntry>();
+  private final Map<Integer,NumericEntry> numerics = new HashMap<>();
+  private final Map<Integer,BinaryEntry> binaries = new HashMap<>();
+  private final Map<Integer,SortedEntry> sorteds = new HashMap<>();
+  private final Map<Integer,SortedSetEntry> sortedSets = new HashMap<>();
+  private final Map<Integer,SortedNumericEntry> sortedNumerics = new HashMap<>();
   private final IndexInput data;
   
   // ram instances we have already loaded
   private final Map<Integer,NumericDocValues> numericInstances = 
-      new HashMap<Integer,NumericDocValues>();
-  private final Map<Integer,BinaryDocValues> binaryInstances =
-      new HashMap<Integer,BinaryDocValues>();
-  private final Map<Integer,SortedDocValues> sortedInstances =
-      new HashMap<Integer,SortedDocValues>();
+      new HashMap<>();
+  private final Map<Integer,BinaryRawValues> binaryInstances =
+      new HashMap<>();
+  private final Map<Integer,SortedRawValues> sortedInstances =
+      new HashMap<>();
   private final Map<Integer,SortedSetRawValues> sortedSetInstances =
-      new HashMap<Integer,SortedSetRawValues>();
-  private final Map<Integer,Bits> docsWithFieldInstances = new HashMap<Integer,Bits>();
+      new HashMap<>();
+  private final Map<Integer,SortedNumericRawValues> sortedNumericInstances =
+      new HashMap<>();
+  private final Map<Integer,Bits> docsWithFieldInstances = new HashMap<>();
   
   private final int maxDoc;
   private final AtomicLong ramBytesUsed;
+  private final int version;
   
   static final byte NUMBER = 0;
   static final byte BYTES = 1;
   static final byte SORTED = 2;
   static final byte SORTED_SET = 3;
+  static final byte SORTED_SET_SINGLETON = 4;
+  static final byte SORTED_NUMERIC = 5;
+  static final byte SORTED_NUMERIC_SINGLETON = 6;
 
-  static final int VERSION_START = 0;
+  static final int VERSION_START = 2;
   static final int VERSION_CURRENT = VERSION_START;
     
   DirectDocValuesProducer(SegmentReadState state, String dataCodec, String dataExtension, String metaCodec, String metaExtension) throws IOException {
     maxDoc = state.segmentInfo.getDocCount();
     String metaName = IndexFileNames.segmentFileName(state.segmentInfo.name, state.segmentSuffix, metaExtension);
     // read in the entries from the metadata file.
-    IndexInput in = state.directory.openInput(metaName, state.context);
+    ChecksumIndexInput in = state.directory.openChecksumInput(metaName, state.context);
     ramBytesUsed = new AtomicLong(RamUsageEstimator.shallowSizeOfInstance(getClass()));
     boolean success = false;
-    final int version;
     try {
       version = CodecUtil.checkHeader(in, metaCodec, 
                                       VERSION_START,
                                       VERSION_CURRENT);
       readFields(in);
 
+      CodecUtil.checkFooter(in);
       success = true;
     } finally {
       if (success) {
@@ -96,16 +107,22 @@ class DirectDocValuesProducer extends DocValuesProducer {
       }
     }
 
+    String dataName = IndexFileNames.segmentFileName(state.segmentInfo.name, state.segmentSuffix, dataExtension);
+    this.data = state.directory.openInput(dataName, state.context);
     success = false;
     try {
-      String dataName = IndexFileNames.segmentFileName(state.segmentInfo.name, state.segmentSuffix, dataExtension);
-      data = state.directory.openInput(dataName, state.context);
       final int version2 = CodecUtil.checkHeader(data, dataCodec, 
                                                  VERSION_START,
                                                  VERSION_CURRENT);
       if (version != version2) {
         throw new CorruptIndexException("Format versions mismatch");
       }
+      
+      // NOTE: data file is too costly to verify checksum against all the bytes on open,
+      // but for now we at least verify proper structure of the checksum footer: which looks
+      // for FOOTER_MAGIC + algorithmID. This is cheap and can detect some forms of corruption
+      // such as file truncation.
+      CodecUtil.retrieveChecksum(data);
 
       success = true;
     } finally {
@@ -152,13 +169,24 @@ class DirectDocValuesProducer extends DocValuesProducer {
     return entry;
   }
 
-  private SortedSetEntry readSortedSetEntry(IndexInput meta) throws IOException {
+  private SortedSetEntry readSortedSetEntry(IndexInput meta, boolean singleton) throws IOException {
     SortedSetEntry entry = new SortedSetEntry();
-    entry.docToOrdAddress = readNumericEntry(meta);
+    if (singleton == false) {
+      entry.docToOrdAddress = readNumericEntry(meta);
+    }
     entry.ords = readNumericEntry(meta);
     entry.values = readBinaryEntry(meta);
     return entry;
   }
+  
+  private SortedNumericEntry readSortedNumericEntry(IndexInput meta, boolean singleton) throws IOException {
+    SortedNumericEntry entry = new SortedNumericEntry();
+    if (singleton == false) {
+      entry.docToAddress = readNumericEntry(meta);
+    }
+    entry.values = readNumericEntry(meta);
+    return entry;
+  }
 
   private void readFields(IndexInput meta) throws IOException {
     int fieldNumber = meta.readVInt();
@@ -169,9 +197,23 @@ class DirectDocValuesProducer extends DocValuesProducer {
       } else if (fieldType == BYTES) {
         binaries.put(fieldNumber, readBinaryEntry(meta));
       } else if (fieldType == SORTED) {
-        sorteds.put(fieldNumber, readSortedEntry(meta));
+        SortedEntry entry = readSortedEntry(meta);
+        sorteds.put(fieldNumber, entry);
+        binaries.put(fieldNumber, entry.values);
       } else if (fieldType == SORTED_SET) {
-        sortedSets.put(fieldNumber, readSortedSetEntry(meta));
+        SortedSetEntry entry = readSortedSetEntry(meta, false);
+        sortedSets.put(fieldNumber, entry);
+        binaries.put(fieldNumber, entry.values);
+      } else if (fieldType == SORTED_SET_SINGLETON) {
+        SortedSetEntry entry = readSortedSetEntry(meta, true);
+        sortedSets.put(fieldNumber, entry);
+        binaries.put(fieldNumber, entry.values);
+      } else if (fieldType == SORTED_NUMERIC) {
+        SortedNumericEntry entry = readSortedNumericEntry(meta, false);
+        sortedNumerics.put(fieldNumber, entry);
+      } else if (fieldType == SORTED_NUMERIC_SINGLETON) {
+        SortedNumericEntry entry = readSortedNumericEntry(meta, true);
+        sortedNumerics.put(fieldNumber, entry);
       } else {
         throw new CorruptIndexException("invalid entry type: " + fieldType + ", input=" + meta);
       }
@@ -184,6 +226,11 @@ class DirectDocValuesProducer extends DocValuesProducer {
     return ramBytesUsed.get();
   }
   
+  @Override
+  public void checkIntegrity() throws IOException {
+    CodecUtil.checksumEntireFile(data);
+  }
+
   @Override
   public synchronized NumericDocValues getNumeric(FieldInfo field) throws IOException {
     NumericDocValues instance = numericInstances.get(field.number);
@@ -263,16 +310,29 @@ class DirectDocValuesProducer extends DocValuesProducer {
 
   @Override
   public synchronized BinaryDocValues getBinary(FieldInfo field) throws IOException {
-    BinaryDocValues instance = binaryInstances.get(field.number);
+    BinaryRawValues instance = binaryInstances.get(field.number);
     if (instance == null) {
       // Lazy load
       instance = loadBinary(binaries.get(field.number));
       binaryInstances.put(field.number, instance);
     }
-    return instance;
+    final byte[] bytes = instance.bytes;
+    final int[] address = instance.address;
+
+    return new BinaryDocValues() {
+      final BytesRef term = new BytesRef();
+
+      @Override
+      public BytesRef get(int docID) {
+        term.bytes = bytes;
+        term.offset = address[docID];
+        term.length = address[docID+1] - term.offset;
+        return term;
+      }
+    };
   }
   
-  private BinaryDocValues loadBinary(BinaryEntry entry) throws IOException {
+  private BinaryRawValues loadBinary(BinaryEntry entry) throws IOException {
     data.seek(entry.offset);
     final byte[] bytes = new byte[entry.numBytes];
     data.readBytes(bytes, 0, entry.numBytes);
@@ -286,32 +346,28 @@ class DirectDocValuesProducer extends DocValuesProducer {
 
     ramBytesUsed.addAndGet(RamUsageEstimator.sizeOf(bytes) + RamUsageEstimator.sizeOf(address));
 
-    return new BinaryDocValues() {
-      @Override
-      public void get(int docID, BytesRef result) {
-        result.bytes = bytes;
-        result.offset = address[docID];
-        result.length = address[docID+1] - result.offset;
-      };
-    };
+    BinaryRawValues values = new BinaryRawValues();
+    values.bytes = bytes;
+    values.address = address;
+    return values;
   }
   
   @Override
-  public synchronized SortedDocValues getSorted(FieldInfo field) throws IOException {
-    SortedDocValues instance = sortedInstances.get(field.number);
-    if (instance == null) {
-      // Lazy load
-      instance = loadSorted(field);
-      sortedInstances.put(field.number, instance);
-    }
-    return instance;
-  }
-
-  private SortedDocValues loadSorted(FieldInfo field) throws IOException {
+  public SortedDocValues getSorted(FieldInfo field) throws IOException {
     final SortedEntry entry = sorteds.get(field.number);
-    final NumericDocValues docToOrd = loadNumeric(entry.docToOrd);
-    final BinaryDocValues values = loadBinary(entry.values);
-
+    SortedRawValues instance;
+    synchronized (this) {
+      instance = sortedInstances.get(field.number);
+      if (instance == null) {
+        // Lazy load
+        instance = loadSorted(field);
+        sortedInstances.put(field.number, instance);
+      }
+    }
+    return newSortedInstance(instance.docToOrd, getBinary(field), entry.values.count);
+  }
+  
+  private SortedDocValues newSortedInstance(final NumericDocValues docToOrd, final BinaryDocValues values, final int count) {
     return new SortedDocValues() {
 
       @Override
@@ -320,13 +376,13 @@ class DirectDocValuesProducer extends DocValuesProducer {
       }
 
       @Override
-      public void lookupOrd(int ord, BytesRef result) {
-        values.get(ord, result);
+      public BytesRef lookupOrd(int ord) {
+        return values.get(ord);
       }
 
       @Override
       public int getValueCount() {
-        return entry.values.count;
+        return count;
       }
 
       // Leave lookupTerm to super's binary search
@@ -335,6 +391,64 @@ class DirectDocValuesProducer extends DocValuesProducer {
     };
   }
 
+  private SortedRawValues loadSorted(FieldInfo field) throws IOException {
+    final SortedEntry entry = sorteds.get(field.number);
+    final NumericDocValues docToOrd = loadNumeric(entry.docToOrd);
+    final SortedRawValues values = new SortedRawValues();
+    values.docToOrd = docToOrd;
+    return values;
+  }
+
+  @Override
+  public synchronized SortedNumericDocValues getSortedNumeric(FieldInfo field) throws IOException {
+    SortedNumericRawValues instance = sortedNumericInstances.get(field.number);
+    final SortedNumericEntry entry = sortedNumerics.get(field.number);
+    if (instance == null) {
+      // Lazy load
+      instance = loadSortedNumeric(entry);
+      sortedNumericInstances.put(field.number, instance);
+    }
+    
+    if (entry.docToAddress == null) {
+      final NumericDocValues single = instance.values;
+      final Bits docsWithField = getMissingBits(field.number, entry.values.missingOffset, entry.values.missingBytes);
+      return DocValues.singleton(single, docsWithField);
+    } else {
+      final NumericDocValues docToAddress = instance.docToAddress;
+      final NumericDocValues values = instance.values;
+      
+      return new SortedNumericDocValues() {
+        int valueStart;
+        int valueLimit;
+        
+        @Override
+        public void setDocument(int doc) {
+          valueStart = (int) docToAddress.get(doc);
+          valueLimit = (int) docToAddress.get(doc+1);
+        }
+        
+        @Override
+        public long valueAt(int index) {
+          return values.get(valueStart + index);
+        }
+        
+        @Override
+        public int count() {
+          return valueLimit - valueStart;
+        }
+      };
+    }
+  }
+  
+  private SortedNumericRawValues loadSortedNumeric(SortedNumericEntry entry) throws IOException {
+    SortedNumericRawValues instance = new SortedNumericRawValues();
+    if (entry.docToAddress != null) {
+      instance.docToAddress = loadNumeric(entry.docToAddress);
+    }
+    instance.values = loadNumeric(entry.values);
+    return instance;
+  }
+
   @Override
   public synchronized SortedSetDocValues getSortedSet(FieldInfo field) throws IOException {
     SortedSetRawValues instance = sortedSetInstances.get(field.number);
@@ -345,51 +459,68 @@ class DirectDocValuesProducer extends DocValuesProducer {
       sortedSetInstances.put(field.number, instance);
     }
 
-    final NumericDocValues docToOrdAddress = instance.docToOrdAddress;
-    final NumericDocValues ords = instance.ords;
-    final BinaryDocValues values = instance.values;
-
-    // Must make a new instance since the iterator has state:
-    return new SortedSetDocValues() {
-      int ordUpto;
-      int ordLimit;
-
-      @Override
-      public long nextOrd() {
-        if (ordUpto == ordLimit) {
-          return NO_MORE_ORDS;
-        } else {
-          return ords.get(ordUpto++);
-        }
-      }
+    if (instance.docToOrdAddress == null) {
+      SortedDocValues sorted = newSortedInstance(instance.ords, getBinary(field), entry.values.count);
+      return DocValues.singleton(sorted);
+    } else {
+      final NumericDocValues docToOrdAddress = instance.docToOrdAddress;
+      final NumericDocValues ords = instance.ords;
+      final BinaryDocValues values = getBinary(field);
       
-      @Override
-      public void setDocument(int docID) {
-        ordUpto = (int) docToOrdAddress.get(docID);
-        ordLimit = (int) docToOrdAddress.get(docID+1);
-      }
-
-      @Override
-      public void lookupOrd(long ord, BytesRef result) {
-        values.get((int) ord, result);
-      }
-
-      @Override
-      public long getValueCount() {
-        return entry.values.count;
-      }
-
-      // Leave lookupTerm to super's binary search
-
-      // Leave termsEnum to super
-    };
+      // Must make a new instance since the iterator has state:
+      return new RandomAccessOrds() {
+        int ordStart;
+        int ordUpto;
+        int ordLimit;
+        
+        @Override
+        public long nextOrd() {
+          if (ordUpto == ordLimit) {
+            return NO_MORE_ORDS;
+          } else {
+            return ords.get(ordUpto++);
+          }
+        }
+        
+        @Override
+        public void setDocument(int docID) {
+          ordStart = ordUpto = (int) docToOrdAddress.get(docID);
+          ordLimit = (int) docToOrdAddress.get(docID+1);
+        }
+        
+        @Override
+        public BytesRef lookupOrd(long ord) {
+          return values.get((int) ord);
+        }
+        
+        @Override
+        public long getValueCount() {
+          return entry.values.count;
+        }
+        
+        @Override
+        public long ordAt(int index) {
+          return ords.get(ordStart + index);
+        }
+        
+        @Override
+        public int cardinality() {
+          return ordLimit - ordStart;
+        }
+        
+        // Leave lookupTerm to super's binary search
+        
+        // Leave termsEnum to super
+      };
+    }
   }
   
   private SortedSetRawValues loadSortedSet(SortedSetEntry entry) throws IOException {
     SortedSetRawValues instance = new SortedSetRawValues();
-    instance.docToOrdAddress = loadNumeric(entry.docToOrdAddress);
+    if (entry.docToOrdAddress != null) {
+      instance.docToOrdAddress = loadNumeric(entry.docToOrdAddress);
+    }
     instance.ords = loadNumeric(entry.ords);
-    instance.values = loadBinary(entry.values);
     return instance;
   }
 
@@ -420,9 +551,11 @@ class DirectDocValuesProducer extends DocValuesProducer {
   public Bits getDocsWithField(FieldInfo field) throws IOException {
     switch(field.getDocValuesType()) {
       case SORTED_SET:
-        return new SortedSetDocsWithField(getSortedSet(field), maxDoc);
+        return DocValues.docsWithValue(getSortedSet(field), maxDoc);
+      case SORTED_NUMERIC:
+        return DocValues.docsWithValue(getSortedNumeric(field), maxDoc);
       case SORTED:
-        return new SortedDocsWithField(getSorted(field), maxDoc);
+        return DocValues.docsWithValue(getSorted(field), maxDoc);
       case BINARY:
         BinaryEntry be = binaries.get(field.number);
         return getMissingBits(field.number, be.missingOffset, be.missingBytes);
@@ -438,11 +571,24 @@ class DirectDocValuesProducer extends DocValuesProducer {
   public void close() throws IOException {
     data.close();
   }
+
+  static class BinaryRawValues {
+    byte[] bytes;
+    int[] address;
+  }
+
+  static class SortedRawValues {
+    NumericDocValues docToOrd;
+  }
   
+  static class SortedNumericRawValues {
+    NumericDocValues docToAddress;
+    NumericDocValues values;
+  }
+
   static class SortedSetRawValues {
     NumericDocValues docToOrdAddress;
     NumericDocValues ords;
-    BinaryDocValues values;
   }
 
   static class NumericEntry {
@@ -477,6 +623,11 @@ class DirectDocValuesProducer extends DocValuesProducer {
     BinaryEntry values;
   }
   
+  static class SortedNumericEntry {
+    NumericEntry docToAddress;
+    NumericEntry values;
+  }
+  
   static class FSTEntry {
     long offset;
     long numOrds;
