@@ -32,12 +32,15 @@ import org.apache.lucene.index.MultiDocValues.OrdinalMap;
 import org.apache.lucene.index.NumericDocValues;
 import org.apache.lucene.index.SegmentWriteState;
 import org.apache.lucene.index.SortedDocValues;
+import org.apache.lucene.index.SortedNumericDocValues;
 import org.apache.lucene.index.SortedSetDocValues;
 import org.apache.lucene.index.TermsEnum;
 import org.apache.lucene.util.ArrayUtil;
 import org.apache.lucene.util.Bits;
 import org.apache.lucene.util.BytesRef;
-import org.apache.lucene.util.OpenBitSet;
+import org.apache.lucene.util.LongBitSet;
+import org.apache.lucene.util.LongValues;
+import org.apache.lucene.util.packed.PackedInts;
 
 /** 
  * Abstract API that consumes numeric, binary and
@@ -93,6 +96,16 @@ public abstract class DocValuesConsumer implements Closeable {
    * @throws IOException if an I/O error occurred.
    */
   public abstract void addSortedField(FieldInfo field, Iterable<BytesRef> values, Iterable<Number> docToOrd) throws IOException;
+  
+  /**
+   * Writes pre-sorted numeric docvalues for a field
+   * @param field field information
+   * @param docToValueCount Iterable of the number of values for each document. A zero
+   *                        count indicates a missing value.
+   * @param values Iterable of numeric values in sorted order (not deduplicated).
+   * @throws IOException if an I/O error occurred.
+   */
+  public abstract void addSortedNumericField(FieldInfo field, Iterable<Number> docToValueCount, Iterable<Number> values) throws IOException;
 
   /**
    * Writes pre-sorted set docvalues for a field
@@ -113,6 +126,8 @@ public abstract class DocValuesConsumer implements Closeable {
    */
   public void mergeNumericField(final FieldInfo fieldInfo, final MergeState mergeState, final List<NumericDocValues> toMerge, final List<Bits> docsWithField) throws IOException {
 
+    mergeState.checkAbort.work(mergeState.segmentInfo.getDocCount());
+
     addNumericField(fieldInfo,
                     new Iterable<Number>() {
                       @Override
@@ -120,7 +135,8 @@ public abstract class DocValuesConsumer implements Closeable {
                         return new Iterator<Number>() {
                           int readerUpto = -1;
                           int docIDUpto;
-                          Long nextValue;
+                          long nextValue;
+                          boolean nextHasValue;
                           AtomicReader currentReader;
                           NumericDocValues currentValues;
                           Bits currentLiveDocs;
@@ -144,7 +160,7 @@ public abstract class DocValuesConsumer implements Closeable {
                             }
                             assert nextIsSet;
                             nextIsSet = false;
-                            return nextValue;
+                            return nextHasValue ? nextValue : null;
                           }
 
                           private boolean setNext() {
@@ -167,10 +183,11 @@ public abstract class DocValuesConsumer implements Closeable {
 
                               if (currentLiveDocs == null || currentLiveDocs.get(docIDUpto)) {
                                 nextIsSet = true;
-                                if (currentDocsWithField.get(docIDUpto)) {
-                                  nextValue = currentValues.get(docIDUpto);
+                                nextValue = currentValues.get(docIDUpto);
+                                if (nextValue == 0 && currentDocsWithField.get(docIDUpto) == false) {
+                                  nextHasValue = false;
                                 } else {
-                                  nextValue = null;
+                                  nextHasValue = true;
                                 }
                                 docIDUpto++;
                                 return true;
@@ -192,6 +209,8 @@ public abstract class DocValuesConsumer implements Closeable {
    */
   public void mergeBinaryField(FieldInfo fieldInfo, final MergeState mergeState, final List<BinaryDocValues> toMerge, final List<Bits> docsWithField) throws IOException {
 
+    mergeState.checkAbort.work(mergeState.segmentInfo.getDocCount());
+
     addBinaryField(fieldInfo,
                    new Iterable<BytesRef>() {
                      @Override
@@ -199,7 +218,7 @@ public abstract class DocValuesConsumer implements Closeable {
                        return new Iterator<BytesRef>() {
                          int readerUpto = -1;
                          int docIDUpto;
-                         BytesRef nextValue = new BytesRef();
+                         BytesRef nextValue;
                          BytesRef nextPointer; // points to null if missing, or nextValue
                          AtomicReader currentReader;
                          BinaryDocValues currentValues;
@@ -248,7 +267,7 @@ public abstract class DocValuesConsumer implements Closeable {
                              if (currentLiveDocs == null || currentLiveDocs.get(docIDUpto)) {
                                nextIsSet = true;
                                if (currentDocsWithField.get(docIDUpto)) {
-                                 currentValues.get(docIDUpto, nextValue);
+                                 nextValue = currentValues.get(docIDUpto);
                                  nextPointer = nextValue;
                                } else {
                                  nextPointer = null;
@@ -265,6 +284,159 @@ public abstract class DocValuesConsumer implements Closeable {
                    });
   }
 
+  /**
+   * Merges the sorted docvalues from <code>toMerge</code>.
+   * <p>
+   * The default implementation calls {@link #addSortedNumericField}, passing
+   * iterables that filter deleted documents.
+   */
+  public void mergeSortedNumericField(FieldInfo fieldInfo, final MergeState mergeState, List<SortedNumericDocValues> toMerge) throws IOException {
+
+    mergeState.checkAbort.work(mergeState.segmentInfo.getDocCount());
+
+    final AtomicReader readers[] = mergeState.readers.toArray(new AtomicReader[toMerge.size()]);
+    final SortedNumericDocValues dvs[] = toMerge.toArray(new SortedNumericDocValues[toMerge.size()]);
+    
+    // step 3: add field
+    addSortedNumericField(fieldInfo,
+        // doc -> value count
+        new Iterable<Number>() {
+          @Override
+          public Iterator<Number> iterator() {
+            return new Iterator<Number>() {
+              int readerUpto = -1;
+              int docIDUpto;
+              int nextValue;
+              AtomicReader currentReader;
+              Bits currentLiveDocs;
+              boolean nextIsSet;
+
+              @Override
+              public boolean hasNext() {
+                return nextIsSet || setNext();
+              }
+
+              @Override
+              public void remove() {
+                throw new UnsupportedOperationException();
+              }
+
+              @Override
+              public Number next() {
+                if (!hasNext()) {
+                  throw new NoSuchElementException();
+                }
+                assert nextIsSet;
+                nextIsSet = false;
+                return nextValue;
+              }
+
+              private boolean setNext() {
+                while (true) {
+                  if (readerUpto == readers.length) {
+                    return false;
+                  }
+
+                  if (currentReader == null || docIDUpto == currentReader.maxDoc()) {
+                    readerUpto++;
+                    if (readerUpto < readers.length) {
+                      currentReader = readers[readerUpto];
+                      currentLiveDocs = currentReader.getLiveDocs();
+                    }
+                    docIDUpto = 0;
+                    continue;
+                  }
+
+                  if (currentLiveDocs == null || currentLiveDocs.get(docIDUpto)) {
+                    nextIsSet = true;
+                    SortedNumericDocValues dv = dvs[readerUpto];
+                    dv.setDocument(docIDUpto);
+                    nextValue = dv.count();
+                    docIDUpto++;
+                    return true;
+                  }
+
+                  docIDUpto++;
+                }
+              }
+            };
+          }
+        },
+        // values
+        new Iterable<Number>() {
+          @Override
+          public Iterator<Number> iterator() {
+            return new Iterator<Number>() {
+              int readerUpto = -1;
+              int docIDUpto;
+              long nextValue;
+              AtomicReader currentReader;
+              Bits currentLiveDocs;
+              boolean nextIsSet;
+              int valueUpto;
+              int valueLength;
+
+              @Override
+              public boolean hasNext() {
+                return nextIsSet || setNext();
+              }
+
+              @Override
+              public void remove() {
+                throw new UnsupportedOperationException();
+              }
+
+              @Override
+              public Number next() {
+                if (!hasNext()) {
+                  throw new NoSuchElementException();
+                }
+                assert nextIsSet;
+                nextIsSet = false;
+                return nextValue;
+              }
+
+              private boolean setNext() {
+                while (true) {
+                  if (readerUpto == readers.length) {
+                    return false;
+                  }
+                  
+                  if (valueUpto < valueLength) {
+                    nextValue = dvs[readerUpto].valueAt(valueUpto);
+                    valueUpto++;
+                    nextIsSet = true;
+                    return true;
+                  }
+
+                  if (currentReader == null || docIDUpto == currentReader.maxDoc()) {
+                    readerUpto++;
+                    if (readerUpto < readers.length) {
+                      currentReader = readers[readerUpto];
+                      currentLiveDocs = currentReader.getLiveDocs();
+                    }
+                    docIDUpto = 0;
+                    continue;
+                  }
+                  
+                  if (currentLiveDocs == null || currentLiveDocs.get(docIDUpto)) {
+                    assert docIDUpto < currentReader.maxDoc();
+                    SortedNumericDocValues dv = dvs[readerUpto];
+                    dv.setDocument(docIDUpto);
+                    valueUpto = 0;
+                    valueLength = dv.count();
+                    docIDUpto++;
+                    continue;
+                  }
+
+                  docIDUpto++;
+                }
+              }
+            };
+          }
+        }
+     );
+  }
 
   /**
    * Merges the sorted docvalues from <code>toMerge</code>.
@@ -273,19 +445,24 @@ public abstract class DocValuesConsumer implements Closeable {
    * an Iterable that merges ordinals and values and filters deleted documents .
    */
   public void mergeSortedField(FieldInfo fieldInfo, final MergeState mergeState, List<SortedDocValues> toMerge) throws IOException {
+
+    mergeState.checkAbort.work(mergeState.segmentInfo.getDocCount());
+
     final AtomicReader readers[] = mergeState.readers.toArray(new AtomicReader[toMerge.size()]);
     final SortedDocValues dvs[] = toMerge.toArray(new SortedDocValues[toMerge.size()]);
     
     // step 1: iterate thru each sub and mark terms still in use
     TermsEnum liveTerms[] = new TermsEnum[dvs.length];
+    long[] weights = new long[liveTerms.length];
     for (int sub = 0; sub < liveTerms.length; sub++) {
       AtomicReader reader = readers[sub];
       SortedDocValues dv = dvs[sub];
       Bits liveDocs = reader.getLiveDocs();
       if (liveDocs == null) {
         liveTerms[sub] = dv.termsEnum();
+        weights[sub] = dv.getValueCount();
       } else {
-        OpenBitSet bitset = new OpenBitSet(dv.getValueCount());
+        LongBitSet bitset = new LongBitSet(dv.getValueCount());
         for (int i = 0; i < reader.maxDoc(); i++) {
           if (liveDocs.get(i)) {
             int ord = dv.getOrd(i);
@@ -295,11 +472,12 @@ public abstract class DocValuesConsumer implements Closeable {
           }
         }
         liveTerms[sub] = new BitsFilteredTermsEnum(dv.termsEnum(), bitset);
+        weights[sub] = bitset.cardinality();
       }
     }
     
     // step 2: create ordinal map (this conceptually does the "merging")
-    final OrdinalMap map = new OrdinalMap(this, liveTerms);
+    final OrdinalMap map = OrdinalMap.build(this, liveTerms, weights, PackedInts.COMPACT);
     
     // step 3: add field
     addSortedField(fieldInfo,
@@ -308,7 +486,6 @@ public abstract class DocValuesConsumer implements Closeable {
           @Override
           public Iterator<BytesRef> iterator() {
             return new Iterator<BytesRef>() {
-              final BytesRef scratch = new BytesRef();
               int currentOrd;
 
               @Override
@@ -323,9 +500,9 @@ public abstract class DocValuesConsumer implements Closeable {
                 }
                 int segmentNumber = map.getFirstSegmentNumber(currentOrd);
                 int segmentOrd = (int)map.getFirstSegmentOrd(currentOrd);
-                dvs[segmentNumber].lookupOrd(segmentOrd, scratch);
+                final BytesRef term = dvs[segmentNumber].lookupOrd(segmentOrd);
                 currentOrd++;
-                return scratch;
+                return term;
               }
 
               @Override
@@ -345,6 +522,7 @@ public abstract class DocValuesConsumer implements Closeable {
               int nextValue;
               AtomicReader currentReader;
               Bits currentLiveDocs;
+              LongValues currentMap;
               boolean nextIsSet;
 
               @Override
@@ -379,6 +557,7 @@ public abstract class DocValuesConsumer implements Closeable {
                     if (readerUpto < readers.length) {
                       currentReader = readers[readerUpto];
                       currentLiveDocs = currentReader.getLiveDocs();
+                      currentMap = map.getGlobalOrds(readerUpto);
                     }
                     docIDUpto = 0;
                     continue;
@@ -387,7 +566,7 @@ public abstract class DocValuesConsumer implements Closeable {
                   if (currentLiveDocs == null || currentLiveDocs.get(docIDUpto)) {
                     nextIsSet = true;
                     int segOrd = dvs[readerUpto].getOrd(docIDUpto);
-                    nextValue = segOrd == -1 ? -1 : (int) map.getGlobalOrd(readerUpto, segOrd);
+                    nextValue = segOrd == -1 ? -1 : (int) currentMap.get(segOrd);
                     docIDUpto++;
                     return true;
                   }
@@ -408,19 +587,24 @@ public abstract class DocValuesConsumer implements Closeable {
    * an Iterable that merges ordinals and values and filters deleted documents .
    */
   public void mergeSortedSetField(FieldInfo fieldInfo, final MergeState mergeState, List<SortedSetDocValues> toMerge) throws IOException {
+
+    mergeState.checkAbort.work(mergeState.segmentInfo.getDocCount());
+
     final AtomicReader readers[] = mergeState.readers.toArray(new AtomicReader[toMerge.size()]);
     final SortedSetDocValues dvs[] = toMerge.toArray(new SortedSetDocValues[toMerge.size()]);
     
     // step 1: iterate thru each sub and mark terms still in use
     TermsEnum liveTerms[] = new TermsEnum[dvs.length];
+    long[] weights = new long[liveTerms.length];
     for (int sub = 0; sub < liveTerms.length; sub++) {
       AtomicReader reader = readers[sub];
       SortedSetDocValues dv = dvs[sub];
       Bits liveDocs = reader.getLiveDocs();
       if (liveDocs == null) {
         liveTerms[sub] = dv.termsEnum();
+        weights[sub] = dv.getValueCount();
       } else {
-        OpenBitSet bitset = new OpenBitSet(dv.getValueCount());
+        LongBitSet bitset = new LongBitSet(dv.getValueCount());
         for (int i = 0; i < reader.maxDoc(); i++) {
           if (liveDocs.get(i)) {
             dv.setDocument(i);
@@ -431,11 +615,12 @@ public abstract class DocValuesConsumer implements Closeable {
           }
         }
         liveTerms[sub] = new BitsFilteredTermsEnum(dv.termsEnum(), bitset);
+        weights[sub] = bitset.cardinality();
       }
     }
     
     // step 2: create ordinal map (this conceptually does the "merging")
-    final OrdinalMap map = new OrdinalMap(this, liveTerms);
+    final OrdinalMap map = OrdinalMap.build(this, liveTerms, weights, PackedInts.COMPACT);
     
     // step 3: add field
     addSortedSetField(fieldInfo,
@@ -444,7 +629,6 @@ public abstract class DocValuesConsumer implements Closeable {
           @Override
           public Iterator<BytesRef> iterator() {
             return new Iterator<BytesRef>() {
-              final BytesRef scratch = new BytesRef();
               long currentOrd;
 
               @Override
@@ -459,9 +643,9 @@ public abstract class DocValuesConsumer implements Closeable {
                 }
                 int segmentNumber = map.getFirstSegmentNumber(currentOrd);
                 long segmentOrd = map.getFirstSegmentOrd(currentOrd);
-                dvs[segmentNumber].lookupOrd(segmentOrd, scratch);
+                final BytesRef term = dvs[segmentNumber].lookupOrd(segmentOrd);
                 currentOrd++;
-                return scratch;
+                return term;
               }
 
               @Override
@@ -548,6 +732,7 @@ public abstract class DocValuesConsumer implements Closeable {
               long nextValue;
               AtomicReader currentReader;
               Bits currentLiveDocs;
+              LongValues currentMap;
               boolean nextIsSet;
               long ords[] = new long[8];
               int ordUpto;
@@ -592,6 +777,7 @@ public abstract class DocValuesConsumer implements Closeable {
                     if (readerUpto < readers.length) {
                       currentReader = readers[readerUpto];
                       currentLiveDocs = currentReader.getLiveDocs();
+                      currentMap = map.getGlobalOrds(readerUpto);
                     }
                     docIDUpto = 0;
                     continue;
@@ -607,7 +793,7 @@ public abstract class DocValuesConsumer implements Closeable {
                       if (ordLength == ords.length) {
                         ords = ArrayUtil.grow(ords, ordLength+1);
                       }
-                      ords[ordLength] = map.getGlobalOrd(readerUpto, ord);
+                      ords[ordLength] = currentMap.get(ord);
                       ordLength++;
                     }
                     docIDUpto++;
@@ -625,9 +811,9 @@ public abstract class DocValuesConsumer implements Closeable {
   
   // TODO: seek-by-ord to nextSetBit
   static class BitsFilteredTermsEnum extends FilteredTermsEnum {
-    final OpenBitSet liveTerms;
+    final LongBitSet liveTerms;
     
-    BitsFilteredTermsEnum(TermsEnum in, OpenBitSet liveTerms) {
+    BitsFilteredTermsEnum(TermsEnum in, LongBitSet liveTerms) {
       super(in, false); // <-- not passing false here wasted about 3 hours of my time!!!!!!!!!!!!!
       assert liveTerms != null;
       this.liveTerms = liveTerms;
@@ -642,4 +828,49 @@ public abstract class DocValuesConsumer implements Closeable {
       }
     }
   }
+  
+  /** Helper: returns true if the given docToValue count contains only at most one value */
+  public static boolean isSingleValued(Iterable<Number> docToValueCount) {
+    for (Number count : docToValueCount) {
+      if (count.longValue() > 1) {
+        return false;
+      }
+    }
+    return true;
+  }
+  
+  /** Helper: returns single-valued view, using {@code missingValue} when count is zero */
+  public static Iterable<Number> singletonView(final Iterable<Number> docToValueCount, final Iterable<Number> values, final Number missingValue) {
+    assert isSingleValued(docToValueCount);
+    return new Iterable<Number>() {
+
+      @Override
+      public Iterator<Number> iterator() {
+        final Iterator<Number> countIterator = docToValueCount.iterator();
+        final Iterator<Number> valuesIterator = values.iterator();
+        return new Iterator<Number>() {
+
+          @Override
+          public boolean hasNext() {
+            return countIterator.hasNext();
+          }
+
+          @Override
+          public Number next() {
+            int count = countIterator.next().intValue();
+            if (count == 0) {
+              return missingValue;
+            } else {
+              return valuesIterator.next();
+            }
+          }
+
+          @Override
+          public void remove() {
+            throw new UnsupportedOperationException();
+          }
+        };
+      }
+    };
+  }
 }
