@@ -21,6 +21,7 @@ import java.io.Closeable;
 import java.io.IOException;
 
 import org.apache.lucene.index.Fields;
+import org.apache.lucene.util.Accountable;
 
 /** Abstract API that produces terms, doc, freq, prox, offset and
  *  payloads postings.  
@@ -28,7 +29,7 @@ import org.apache.lucene.index.Fields;
  * @lucene.experimental
  */
 
-public abstract class FieldsProducer extends Fields implements Closeable {
+public abstract class FieldsProducer extends Fields implements Closeable, Accountable {
   /** Sole constructor. (For invocation by subclass 
    *  constructors, typically implicit.) */
   protected FieldsProducer() {
@@ -37,6 +38,12 @@ public abstract class FieldsProducer extends Fields implements Closeable {
   @Override
   public abstract void close() throws IOException;
   
-  /** Returns approximate RAM bytes used */
-  public abstract long ramBytesUsed();
+  /** 
+   * Checks consistency of this reader.
+   * <p>
+   * Note that this may be costly in terms of I/O, e.g. 
+   * may involve computing a checksum value against large data files.
+   * @lucene.internal
+   */
+  public abstract void checkIntegrity() throws IOException;
 }
