@@ -59,8 +59,9 @@ import org.apache.lucene.search.FieldCache.Shorts;
 import org.apache.lucene.store.Directory;
 import org.apache.lucene.util.Bits;
 import org.apache.lucene.util.BytesRef;
+import org.apache.lucene.util.IOUtils;
 import org.apache.lucene.util.LuceneTestCase;
-import org.apache.lucene.util._TestUtil;
+import org.apache.lucene.util.TestUtil;
 import org.junit.AfterClass;
 import org.junit.BeforeClass;
 
@@ -77,7 +78,7 @@ public class TestFieldCache extends LuceneTestCase {
     NUM_DOCS = atLeast(500);
     NUM_ORDS = atLeast(2);
     directory = newDirectory();
-    RandomIndexWriter writer= new RandomIndexWriter(random(), directory, newIndexWriterConfig(TEST_VERSION_CURRENT, new MockAnalyzer(random())).setMergePolicy(newLogMergePolicy()));
+    RandomIndexWriter writer= new RandomIndexWriter(random(), directory, newIndexWriterConfig(new MockAnalyzer(random())).setMergePolicy(newLogMergePolicy()));
     long theLong = Long.MAX_VALUE;
     double theDouble = Double.MAX_VALUE;
     byte theByte = Byte.MAX_VALUE;
@@ -141,11 +142,12 @@ public class TestFieldCache extends LuceneTestCase {
     try {
       FieldCache cache = FieldCache.DEFAULT;
       ByteArrayOutputStream bos = new ByteArrayOutputStream(1024);
-      cache.setInfoStream(new PrintStream(bos, false, "UTF-8"));
+      cache.setInfoStream(new PrintStream(bos, false, IOUtils.UTF_8));
       cache.getDoubles(reader, "theDouble", false);
       cache.getFloats(reader, "theDouble", false);
-      assertTrue(bos.toString("UTF-8").indexOf("WARNING") != -1);
+      assertTrue(bos.toString(IOUtils.UTF_8).indexOf("WARNING") != -1);
     } finally {
+      FieldCache.DEFAULT.setInfoStream(null);
       FieldCache.DEFAULT.purgeAllCaches();
     }
   }
@@ -212,16 +214,13 @@ public class TestFieldCache extends LuceneTestCase {
 
     // getTermsIndex
     SortedDocValues termsIndex = cache.getTermsIndex(reader, "theRandomUnicodeString");
-    assertSame("Second request to cache return same array", termsIndex, cache.getTermsIndex(reader, "theRandomUnicodeString"));
-    final BytesRef br = new BytesRef();
     for (int i = 0; i < NUM_DOCS; i++) {
       final BytesRef term;
       final int ord = termsIndex.getOrd(i);
       if (ord == -1) {
         term = null;
       } else {
-        termsIndex.lookupOrd(ord, br);
-        term = br;
+        term = termsIndex.lookupOrd(ord);
       }
       final String s = term == null ? null : term.utf8ToString();
       assertTrue("for doc " + i + ": " + s + " does not equal: " + unicodeStrings[i], unicodeStrings[i] == null || unicodeStrings[i].equals(s));
@@ -230,10 +229,9 @@ public class TestFieldCache extends LuceneTestCase {
     int nTerms = termsIndex.getValueCount();
 
     TermsEnum tenum = termsIndex.termsEnum();
-    BytesRef val = new BytesRef();
     for (int i=0; i<nTerms; i++) {
       BytesRef val1 = tenum.next();
-      termsIndex.lookupOrd(i, val);
+      BytesRef val = termsIndex.lookupOrd(i);
       // System.out.println("i="+i);
       assertEquals(val, val1);
     }
@@ -242,13 +240,13 @@ public class TestFieldCache extends LuceneTestCase {
     int num = atLeast(100);
     for (int i = 0; i < num; i++) {
       int k = random().nextInt(nTerms);
-      termsIndex.lookupOrd(k, val);
+      BytesRef val = BytesRef.deepCopyOf(termsIndex.lookupOrd(k));
       assertEquals(TermsEnum.SeekStatus.FOUND, tenum.seekCeil(val));
       assertEquals(val, tenum.term());
     }
 
     for(int i=0;i<nTerms;i++) {
-      termsIndex.lookupOrd(i, val);
+      BytesRef val = BytesRef.deepCopyOf(termsIndex.lookupOrd(i));
       assertEquals(TermsEnum.SeekStatus.FOUND, tenum.seekCeil(val));
       assertEquals(val, tenum.term());
     }
@@ -258,10 +256,9 @@ public class TestFieldCache extends LuceneTestCase {
 
     // getTerms
     BinaryDocValues terms = cache.getTerms(reader, "theRandomUnicodeString", true);
-    assertSame("Second request to cache return same array", terms, cache.getTerms(reader, "theRandomUnicodeString", true));
     Bits bits = cache.getDocsWithField(reader, "theRandomUnicodeString");
     for (int i = 0; i < NUM_DOCS; i++) {
-      terms.get(i, br);
+      BytesRef br = terms.get(i);
       final BytesRef term;
       if (!bits.get(i)) {
         term = null;
@@ -285,7 +282,7 @@ public class TestFieldCache extends LuceneTestCase {
     for (int i = 0; i < NUM_DOCS; i++) {
       termOrds.setDocument(i);
       // This will remove identical terms. A DocTermOrds doesn't return duplicate ords for a docId
-      List<BytesRef> values = new ArrayList<BytesRef>(new LinkedHashSet<BytesRef>(Arrays.asList(multiValued[i])));
+      List<BytesRef> values = new ArrayList<>(new LinkedHashSet<>(Arrays.asList(multiValued[i])));
       for (BytesRef v : values) {
         if (v == null) {
           // why does this test use null values... instead of an empty list: confusing
@@ -293,8 +290,7 @@ public class TestFieldCache extends LuceneTestCase {
         }
         long ord = termOrds.nextOrd();
         assert ord != SortedSetDocValues.NO_MORE_ORDS;
-        BytesRef scratch = new BytesRef();
-        termOrds.lookupOrd(ord, scratch);
+        BytesRef scratch = termOrds.lookupOrd(ord);
         assertEquals(v, scratch);
       }
       assertEquals(SortedSetDocValues.NO_MORE_ORDS, termOrds.nextOrd());
@@ -309,7 +305,7 @@ public class TestFieldCache extends LuceneTestCase {
 
   public void testEmptyIndex() throws Exception {
     Directory dir = newDirectory();
-    IndexWriter writer= new IndexWriter(dir, newIndexWriterConfig( TEST_VERSION_CURRENT, new MockAnalyzer(random())).setMaxBufferedDocs(500));
+    IndexWriter writer= new IndexWriter(dir, newIndexWriterConfig(new MockAnalyzer(random())).setMaxBufferedDocs(500));
     writer.close();
     IndexReader r = DirectoryReader.open(dir);
     AtomicReader reader = SlowCompositeReaderWrapper.wrap(r);
@@ -328,10 +324,10 @@ public class TestFieldCache extends LuceneTestCase {
         s = unicodeStrings[random().nextInt(i)];
       }
       if (s == null) {
-        s = _TestUtil.randomUnicodeString(random());
+        s = TestUtil.randomUnicodeString(random());
       }
     } else {
-      s = _TestUtil.randomUnicodeString(random());
+      s = TestUtil.randomUnicodeString(random());
     }
     return s;
   }
@@ -446,7 +442,7 @@ public class TestFieldCache extends LuceneTestCase {
   public void testDocValuesIntegration() throws Exception {
     assumeTrue("3.x does not support docvalues", defaultCodecSupportsDocValues());
     Directory dir = newDirectory();
-    IndexWriterConfig iwc = newIndexWriterConfig(TEST_VERSION_CURRENT, null);
+    IndexWriterConfig iwc = newIndexWriterConfig(null);
     RandomIndexWriter iw = new RandomIndexWriter(random(), dir, iwc);
     Document doc = new Document();
     doc.add(new BinaryDocValuesField("binary", new BytesRef("binary value")));
@@ -461,8 +457,6 @@ public class TestFieldCache extends LuceneTestCase {
     iw.close();
     AtomicReader ar = getOnlySegmentReader(ir);
     
-    BytesRef scratch = new BytesRef();
-    
     // Binary type: can be retrieved via getTerms()
     try {
       FieldCache.DEFAULT.getInts(ar, "binary", false);
@@ -470,7 +464,7 @@ public class TestFieldCache extends LuceneTestCase {
     } catch (IllegalStateException expected) {}
     
     BinaryDocValues binary = FieldCache.DEFAULT.getTerms(ar, "binary", true);
-    binary.get(0, scratch);
+    BytesRef scratch = binary.get(0);
     assertEquals("binary value", scratch.utf8ToString());
     
     try {
@@ -503,13 +497,13 @@ public class TestFieldCache extends LuceneTestCase {
     } catch (IllegalStateException expected) {}
     
     binary = FieldCache.DEFAULT.getTerms(ar, "sorted", true);
-    binary.get(0, scratch);
+    scratch = binary.get(0);
     assertEquals("sorted value", scratch.utf8ToString());
     
     SortedDocValues sorted = FieldCache.DEFAULT.getTermsIndex(ar, "sorted");
     assertEquals(0, sorted.getOrd(0));
     assertEquals(1, sorted.getValueCount());
-    sorted.get(0, scratch);
+    scratch = sorted.get(0);
     assertEquals("sorted value", scratch.utf8ToString());
     
     SortedSetDocValues sortedSet = FieldCache.DEFAULT.getDocTermOrds(ar, "sorted");
@@ -617,14 +611,13 @@ public class TestFieldCache extends LuceneTestCase {
     Doubles doubles = cache.getDoubles(ar, "bogusdoubles", true);
     assertEquals(0, doubles.get(0), 0.0D);
     
-    BytesRef scratch = new BytesRef();
     BinaryDocValues binaries = cache.getTerms(ar, "bogusterms", true);
-    binaries.get(0, scratch);
+    BytesRef scratch = binaries.get(0);
     assertEquals(0, scratch.length);
     
     SortedDocValues sorted = cache.getTermsIndex(ar, "bogustermsindex");
     assertEquals(-1, sorted.getOrd(0));
-    sorted.get(0, scratch);
+    scratch = sorted.get(0);
     assertEquals(0, scratch.length);
     
     SortedSetDocValues sortedSet = cache.getDocTermOrds(ar, "bogusmultivalued");
@@ -682,14 +675,13 @@ public class TestFieldCache extends LuceneTestCase {
     Doubles doubles = cache.getDoubles(ar, "bogusdoubles", true);
     assertEquals(0, doubles.get(0), 0.0D);
     
-    BytesRef scratch = new BytesRef();
     BinaryDocValues binaries = cache.getTerms(ar, "bogusterms", true);
-    binaries.get(0, scratch);
+    BytesRef scratch = binaries.get(0);
     assertEquals(0, scratch.length);
     
     SortedDocValues sorted = cache.getTermsIndex(ar, "bogustermsindex");
     assertEquals(-1, sorted.getOrd(0));
-    sorted.get(0, scratch);
+    scratch = sorted.get(0);
     assertEquals(0, scratch.length);
     
     SortedSetDocValues sortedSet = cache.getDocTermOrds(ar, "bogusmultivalued");
@@ -708,13 +700,13 @@ public class TestFieldCache extends LuceneTestCase {
   // Make sure that the use of GrowableWriter doesn't prevent from using the full long range
   public void testLongFieldCache() throws IOException {
     Directory dir = newDirectory();
-    IndexWriterConfig cfg = newIndexWriterConfig(TEST_VERSION_CURRENT, new MockAnalyzer(random()));
+    IndexWriterConfig cfg = newIndexWriterConfig(new MockAnalyzer(random()));
     cfg.setMergePolicy(newLogMergePolicy());
     RandomIndexWriter iw = new RandomIndexWriter(random(), dir, cfg);
     Document doc = new Document();
     LongField field = new LongField("f", 0L, Store.YES);
     doc.add(field);
-    final long[] values = new long[_TestUtil.nextInt(random(), 1, 10)];
+    final long[] values = new long[TestUtil.nextInt(random(), 1, 10)];
     for (int i = 0; i < values.length; ++i) {
       final long v;
       switch (random().nextInt(10)) {
@@ -728,7 +720,7 @@ public class TestFieldCache extends LuceneTestCase {
           v = Long.MAX_VALUE;
           break;
         default:
-          v = _TestUtil.nextLong(random(), -10, 10);
+          v = TestUtil.nextLong(random(), -10, 10);
           break;
       }
       values[i] = v;
@@ -754,13 +746,13 @@ public class TestFieldCache extends LuceneTestCase {
   // Make sure that the use of GrowableWriter doesn't prevent from using the full int range
   public void testIntFieldCache() throws IOException {
     Directory dir = newDirectory();
-    IndexWriterConfig cfg = newIndexWriterConfig(TEST_VERSION_CURRENT, new MockAnalyzer(random()));
+    IndexWriterConfig cfg = newIndexWriterConfig(new MockAnalyzer(random()));
     cfg.setMergePolicy(newLogMergePolicy());
     RandomIndexWriter iw = new RandomIndexWriter(random(), dir, cfg);
     Document doc = new Document();
     IntField field = new IntField("f", 0, Store.YES);
     doc.add(field);
-    final int[] values = new int[_TestUtil.nextInt(random(), 1, 10)];
+    final int[] values = new int[TestUtil.nextInt(random(), 1, 10)];
     for (int i = 0; i < values.length; ++i) {
       final int v;
       switch (random().nextInt(10)) {
@@ -774,7 +766,7 @@ public class TestFieldCache extends LuceneTestCase {
           v = Integer.MAX_VALUE;
           break;
         default:
-          v = _TestUtil.nextInt(random(), -10, 10);
+          v = TestUtil.nextInt(random(), -10, 10);
           break;
       }
       values[i] = v;
