@@ -22,6 +22,7 @@ import java.lang.reflect.Constructor;
 import java.lang.reflect.InvocationTargetException;
 import java.lang.reflect.Method;
 import java.lang.reflect.Modifier;
+import java.nio.charset.StandardCharsets;
 import java.text.ParseException;
 import java.util.Arrays;
 import java.util.Collections;
@@ -83,7 +84,7 @@ public class JavascriptCompiler {
     }
   }
   
-  private static final int CLASSFILE_VERSION = Opcodes.V1_6;
+  private static final int CLASSFILE_VERSION = Opcodes.V1_7;
   
   // We use the same class name for all generated classes as they all have their own class loader.
   // The source code is displayed as "source file name" in stack trace.
@@ -108,7 +109,7 @@ public class JavascriptCompiler {
   private static final int MAX_SOURCE_LENGTH = 16384;
   
   private final String sourceText;
-  private final Map<String, Integer> externalsMap = new LinkedHashMap<String, Integer>();
+  private final Map<String, Integer> externalsMap = new LinkedHashMap<>();
   private final ClassWriter classWriter = new ClassWriter(ClassWriter.COMPUTE_FRAMES | ClassWriter.COMPUTE_MAXS);
   private GeneratorAdapter gen;
   
@@ -259,8 +260,12 @@ public class JavascriptCompiler {
         
         gen.cast(Type.DOUBLE_TYPE, expected);
         break;
-      case JavascriptParser.NAMESPACE_ID:
+      case JavascriptParser.VARIABLE:
         int index;
+
+        // normalize quotes
+        text = normalizeQuotes(text);
+
         
         if (externalsMap.containsKey(text)) {
           index = externalsMap.get(text);
@@ -496,6 +501,46 @@ public class JavascriptCompiler {
       throw exception;
     }
   }
+
+  private static String normalizeQuotes(String text) {
+    StringBuilder out = new StringBuilder(text.length());
+    boolean inDoubleQuotes = false;
+    for (int i = 0; i < text.length(); ++i) {
+      char c = text.charAt(i);
+      if (c == '\\') {
+        c = text.charAt(++i);
+        if (c == '\\') {
+          out.append('\\'); // re-escape the backslash
+        }
+        // no escape for double quote
+      } else if (c == '\'') {
+        if (inDoubleQuotes) {
+          // escape in output
+          out.append('\\');
+        } else {
+          int j = findSingleQuoteStringEnd(text, i);
+          out.append(text, i, j); // copy up to end quote (leave end for append below)
+          i = j;
+        }
+      } else if (c == '"') {
+        c = '\''; // change beginning/ending doubles to singles
+        inDoubleQuotes = !inDoubleQuotes;
+      }
+      out.append(c);
+    }
+    return out.toString();
+  }
+
+  private static int findSingleQuoteStringEnd(String text, int start) {
+    ++start; // skip beginning
+    while (text.charAt(start) != '\'') {
+      if (text.charAt(start) == '\\') {
+        ++start; // blindly consume escape value
+      }
+      ++start;
+    }
+    return start;
+  }
   
   /** 
    * The default set of functions available to expressions.
@@ -505,13 +550,13 @@ public class JavascriptCompiler {
    */
   public static final Map<String,Method> DEFAULT_FUNCTIONS;
   static {
-    Map<String,Method> map = new HashMap<String,Method>();
-    Reader in = null;
+    Map<String,Method> map = new HashMap<>();
     try {
       final Properties props = new Properties();
-      in = IOUtils.getDecodingReader(JavascriptCompiler.class,
-        JavascriptCompiler.class.getSimpleName() + ".properties", IOUtils.CHARSET_UTF_8);
-      props.load(in);
+      try (Reader in = IOUtils.getDecodingReader(JavascriptCompiler.class,
+        JavascriptCompiler.class.getSimpleName() + ".properties", StandardCharsets.UTF_8)) {
+        props.load(in);
+      }
       for (final String call : props.stringPropertyNames()) {
         final String[] vals = props.getProperty(call).split(",");
         if (vals.length != 3) {
@@ -526,14 +571,8 @@ public class JavascriptCompiler {
         checkFunction(method, JavascriptCompiler.class.getClassLoader());
         map.put(call, method);
       }
-    } catch (NoSuchMethodException e) {
+    } catch (NoSuchMethodException | ClassNotFoundException | IOException e) {
       throw new Error("Cannot resolve function", e);
-    } catch (ClassNotFoundException e) {
-      throw new Error("Cannot resolve function", e);
-    } catch (IOException e) {
-      throw new Error("Cannot resolve function", e);
-    } finally {
-      IOUtils.closeWhileHandlingException(in);
     }
     DEFAULT_FUNCTIONS = Collections.unmodifiableMap(map);
   }
