@@ -30,7 +30,7 @@ import org.apache.lucene.search.grouping.term.TermGroupFacetCollector;
 import org.apache.lucene.store.Directory;
 import org.apache.lucene.store.FSDirectory;
 import org.apache.lucene.util.BytesRef;
-import org.apache.lucene.util._TestUtil;
+import org.apache.lucene.util.TestUtil;
 
 import java.io.IOException;
 import java.util.*;
@@ -46,8 +46,7 @@ public class GroupFacetCollectorTest extends AbstractGroupingTestCase {
     RandomIndexWriter w = new RandomIndexWriter(
         random(),
         dir,
-        newIndexWriterConfig(TEST_VERSION_CURRENT,
-            new MockAnalyzer(random())).setMergePolicy(newLogMergePolicy()));
+        newIndexWriterConfig(new MockAnalyzer(random())).setMergePolicy(newLogMergePolicy()));
     boolean canUseDV = !"Lucene3x".equals(w.w.getConfig().getCodec().getName());
     boolean useDv = canUseDV && random().nextBoolean();
 
@@ -272,8 +271,7 @@ public class GroupFacetCollectorTest extends AbstractGroupingTestCase {
     RandomIndexWriter w = new RandomIndexWriter(
         random(),
         dir,
-        newIndexWriterConfig(TEST_VERSION_CURRENT,
-            new MockAnalyzer(random())).setMergePolicy(NoMergePolicy.COMPOUND_FILES));
+        newIndexWriterConfig(new MockAnalyzer(random())).setMergePolicy(NoMergePolicy.INSTANCE));
     boolean useDv = false;
 
     // Cannot assert this since we use NoMergePolicy:
@@ -303,7 +301,6 @@ public class GroupFacetCollectorTest extends AbstractGroupingTestCase {
     doc = new Document();
     addField(doc, groupField, "a", useDv);
     doc.add(new StringField("airport", "dus", Field.Store.NO));
-
     w.addDocument(doc);
 
     // 4
@@ -359,7 +356,7 @@ public class GroupFacetCollectorTest extends AbstractGroupingTestCase {
 
   public void testRandom() throws Exception {
     Random random = random();
-    int numberOfRuns = _TestUtil.nextInt(random, 3, 6);
+    int numberOfRuns = TestUtil.nextInt(random, 3, 6);
     for (int indexIter = 0; indexIter < numberOfRuns; indexIter++) {
       boolean multipleFacetsPerDocument = random.nextBoolean();
       IndexContext context = createIndexContext(multipleFacetsPerDocument);
@@ -465,23 +462,23 @@ public class GroupFacetCollectorTest extends AbstractGroupingTestCase {
 
   private IndexContext createIndexContext(boolean multipleFacetValuesPerDocument) throws IOException {
     final Random random = random();
-    final int numDocs = _TestUtil.nextInt(random, 138, 1145) * RANDOM_MULTIPLIER;
-    final int numGroups = _TestUtil.nextInt(random, 1, numDocs / 4);
-    final int numFacets = _TestUtil.nextInt(random, 1, numDocs / 6);
+    final int numDocs = TestUtil.nextInt(random, 138, 1145) * RANDOM_MULTIPLIER;
+    final int numGroups = TestUtil.nextInt(random, 1, numDocs / 4);
+    final int numFacets = TestUtil.nextInt(random, 1, numDocs / 6);
 
     if (VERBOSE) {
       System.out.println("TEST: numDocs=" + numDocs + " numGroups=" + numGroups);
     }
 
-    final List<String> groups = new ArrayList<String>();
+    final List<String> groups = new ArrayList<>();
     for (int i = 0; i < numGroups; i++) {
       groups.add(generateRandomNonEmptyString());
     }
-    final List<String> facetValues = new ArrayList<String>();
+    final List<String> facetValues = new ArrayList<>();
     for (int i = 0; i < numFacets; i++) {
       facetValues.add(generateRandomNonEmptyString());
     }
-    final String[] contentBrs = new String[_TestUtil.nextInt(random, 2, 20)];
+    final String[] contentBrs = new String[TestUtil.nextInt(random, 2, 20)];
     if (VERBOSE) {
       System.out.println("TEST: create fake content");
     }
@@ -496,10 +493,7 @@ public class GroupFacetCollectorTest extends AbstractGroupingTestCase {
     RandomIndexWriter writer = new RandomIndexWriter(
         random,
         dir,
-        newIndexWriterConfig(
-            TEST_VERSION_CURRENT,
-            new MockAnalyzer(random)
-        )
+        newIndexWriterConfig(new MockAnalyzer(random))
     );
     boolean canUseDV = !"Lucene3x".equals(writer.w.getConfig().getCodec().getName());
     boolean useDv = canUseDV && !multipleFacetValuesPerDocument && random.nextBoolean();
@@ -540,7 +534,7 @@ public class GroupFacetCollectorTest extends AbstractGroupingTestCase {
     docNoFacet.add(content);
     docNoGroupNoFacet.add(content);
 
-    NavigableSet<String> uniqueFacetValues = new TreeSet<String>(new Comparator<String>() {
+    NavigableSet<String> uniqueFacetValues = new TreeSet<>(new Comparator<String>() {
 
       @Override
       public int compare(String a, String b) {
@@ -556,7 +550,7 @@ public class GroupFacetCollectorTest extends AbstractGroupingTestCase {
       }
 
     });
-    Map<String, Map<String, Set<String>>> searchTermToFacetToGroups = new HashMap<String, Map<String, Set<String>>>();
+    Map<String, Map<String, Set<String>>> searchTermToFacetToGroups = new HashMap<>();
     int facetWithMostGroups = 0;
     for (int i = 0; i < numDocs; i++) {
       final String groupValue;
@@ -578,7 +572,7 @@ public class GroupFacetCollectorTest extends AbstractGroupingTestCase {
       }
       Map<String, Set<String>> facetToGroups = searchTermToFacetToGroups.get(contentStr);
 
-      List<String> facetVals = new ArrayList<String>();
+      List<String> facetVals = new ArrayList<>();
       if (useDv || random.nextInt(24) != 18) {
         if (useDv) {
           String facetValue = facetValues.get(random.nextInt(facetValues.size()));
@@ -656,14 +650,14 @@ public class GroupFacetCollectorTest extends AbstractGroupingTestCase {
   private GroupedFacetResult createExpectedFacetResult(String searchTerm, IndexContext context, int offset, int limit, int minCount, final boolean orderByCount, String facetPrefix) {
     Map<String, Set<String>> facetGroups = context.searchTermToFacetGroups.get(searchTerm);
     if (facetGroups == null) {
-      facetGroups = new HashMap<String, Set<String>>();
+      facetGroups = new HashMap<>();
     }
 
     int totalCount = 0;
     int totalMissCount = 0;
     Set<String> facetValues;
     if (facetPrefix != null) {
-      facetValues = new HashSet<String>();
+      facetValues = new HashSet<>();
       for (String facetValue : context.facetValues) {
         if (facetValue != null && facetValue.startsWith(facetPrefix)) {
           facetValues.add(facetValue);
@@ -673,7 +667,7 @@ public class GroupFacetCollectorTest extends AbstractGroupingTestCase {
       facetValues = context.facetValues;
     }
 
-    List<TermGroupFacetCollector.FacetEntry> entries = new ArrayList<TermGroupFacetCollector.FacetEntry>(facetGroups.size());
+    List<TermGroupFacetCollector.FacetEntry> entries = new ArrayList<>(facetGroups.size());
     // also includes facets with count 0
     for (String facetValue : facetValues) {
       if (facetValue == null) {
