@@ -22,6 +22,7 @@ import java.io.IOException;
 import java.io.InputStream;
 import java.io.InputStreamReader;
 import java.io.Reader;
+import java.nio.charset.StandardCharsets;
 import java.util.HashSet;
 import java.util.Set;
 
@@ -39,6 +40,8 @@ import org.apache.lucene.analysis.tokenattributes.CharTermAttribute;
 import org.apache.lucene.codecs.lucene41.Lucene41PostingsFormat;
 import org.apache.lucene.document.Document;
 import org.apache.lucene.document.Field;
+import org.apache.lucene.document.FieldType;
+import org.apache.lucene.document.TextField;
 import org.apache.lucene.index.AtomicReader;
 import org.apache.lucene.index.CompositeReader;
 import org.apache.lucene.index.DirectoryReader;
@@ -49,7 +52,6 @@ import org.apache.lucene.index.IndexReader;
 import org.apache.lucene.index.IndexWriter;
 import org.apache.lucene.index.IndexWriterConfig;
 import org.apache.lucene.index.IndexableField;
-import org.apache.lucene.index.MultiFields;
 import org.apache.lucene.index.NumericDocValues;
 import org.apache.lucene.index.SlowCompositeReaderWrapper;
 import org.apache.lucene.index.Term;
@@ -74,14 +76,16 @@ import org.apache.lucene.util.IOUtils;
 import org.apache.lucene.util.LineFileDocs;
 import org.apache.lucene.util.RamUsageEstimator;
 import org.apache.lucene.util.RecyclingByteBlockAllocator;
-import org.apache.lucene.util._TestUtil;
+import org.apache.lucene.util.TestUtil;
+
+import static org.hamcrest.CoreMatchers.equalTo;
 
 /**
  * Verifies that Lucene MemoryIndex and RAMDirectory have the same behaviour,
  * returning the same results for queries on some randomish indexes.
  */
 public class MemoryIndexTest extends BaseTokenStreamTestCase {
-  private Set<String> queries = new HashSet<String>();
+  private Set<String> queries = new HashSet<>();
   
   public static final int ITERATIONS = 100 * RANDOM_MULTIPLIER;
 
@@ -96,9 +100,9 @@ public class MemoryIndexTest extends BaseTokenStreamTestCase {
    * read a set of queries from a resource file
    */
   private Set<String> readQueries(String resource) throws IOException {
-    Set<String> queries = new HashSet<String>();
+    Set<String> queries = new HashSet<>();
     InputStream stream = getClass().getResourceAsStream(resource);
-    BufferedReader reader = new BufferedReader(new InputStreamReader(stream, "UTF-8"));
+    BufferedReader reader = new BufferedReader(new InputStreamReader(stream, StandardCharsets.UTF_8));
     String line = null;
     while ((line = reader.readLine()) != null) {
       line = line.trim();
@@ -146,7 +150,7 @@ public class MemoryIndexTest extends BaseTokenStreamTestCase {
     Directory ramdir = new RAMDirectory();
     Analyzer analyzer = randomAnalyzer();
     IndexWriter writer = new IndexWriter(ramdir,
-                                         new IndexWriterConfig(TEST_VERSION_CURRENT, analyzer).setCodec(_TestUtil.alwaysPostingsFormat(new Lucene41PostingsFormat())));
+                                         new IndexWriterConfig(TEST_VERSION_CURRENT, analyzer).setCodec(TestUtil.alwaysPostingsFormat(new Lucene41PostingsFormat())));
     Document doc = new Document();
     Field field1 = newTextField("foo", fooField.toString(), Field.Store.NO);
     Field field2 = newTextField("term", termField.toString(), Field.Store.NO);
@@ -158,14 +162,6 @@ public class MemoryIndexTest extends BaseTokenStreamTestCase {
     memory.addField("foo", fooField.toString(), analyzer);
     memory.addField("term", termField.toString(), analyzer);
     
-    if (VERBOSE) {
-      System.out.println("Random MemoryIndex:\n" + memory.toString());
-      System.out.println("Same index as RAMDirectory: " +
-        RamUsageEstimator.humanReadableUnits(RamUsageEstimator.sizeOf(ramdir)));
-      System.out.println();
-    } else {
-      assertTrue(memory.getMemorySize() > 0L);
-    }
     AtomicReader reader = (AtomicReader) memory.createSearcher().getIndexReader();
     DirectoryReader competitor = DirectoryReader.open(ramdir);
     duellReaders(competitor, reader);
@@ -243,7 +239,7 @@ public class MemoryIndexTest extends BaseTokenStreamTestCase {
     IndexReader reader = DirectoryReader.open(ramdir);
     IndexSearcher ram = newSearcher(reader);
     IndexSearcher mem = memory.createSearcher();
-    QueryParser qp = new QueryParser(TEST_VERSION_CURRENT, "foo", analyzer);
+    QueryParser qp = new QueryParser("foo", analyzer);
     for (String query : queries) {
       TopDocs ramDocs = ram.search(qp.parse(query), 1);
       TopDocs memDocs = mem.search(qp.parse(query), 1);
@@ -311,7 +307,7 @@ public class MemoryIndexTest extends BaseTokenStreamTestCase {
       return TEST_TERMS[random().nextInt(TEST_TERMS.length)];
     } else {
       // return a random unicode term
-      return _TestUtil.randomUnicodeString(random());
+      return TestUtil.randomUnicodeString(random());
     }
   }
   
@@ -320,7 +316,7 @@ public class MemoryIndexTest extends BaseTokenStreamTestCase {
     MemoryIndex memory = new MemoryIndex(random().nextBoolean(),  random().nextInt(50) * 1024 * 1024);
     memory.addField("foo", "bar", analyzer);
     AtomicReader reader = (AtomicReader) memory.createSearcher().getIndexReader();
-    DocsEnum disi = _TestUtil.docs(random(), reader, "foo", new BytesRef("bar"), null, null, DocsEnum.FLAG_NONE);
+    DocsEnum disi = TestUtil.docs(random(), reader, "foo", new BytesRef("bar"), null, null, DocsEnum.FLAG_NONE);
     int docid = disi.docID();
     assertEquals(-1, docid);
     assertTrue(disi.nextDoc() != DocIdSetIterator.NO_MORE_DOCS);
@@ -374,7 +370,7 @@ public class MemoryIndexTest extends BaseTokenStreamTestCase {
   // LUCENE-3831
   public void testNullPointerException() throws IOException {
     RegexpQuery regex = new RegexpQuery(new Term("field", "worl."));
-    SpanQuery wrappedquery = new SpanMultiTermQueryWrapper<RegexpQuery>(regex);
+    SpanQuery wrappedquery = new SpanMultiTermQueryWrapper<>(regex);
         
     MemoryIndex mindex = new MemoryIndex(random().nextBoolean(),  random().nextInt(50) * 1024 * 1024);
     mindex.addField("field", new MockAnalyzer(random()).tokenStream("field", "hello there"));
@@ -386,7 +382,7 @@ public class MemoryIndexTest extends BaseTokenStreamTestCase {
   // LUCENE-3831
   public void testPassesIfWrapped() throws IOException {
     RegexpQuery regex = new RegexpQuery(new Term("field", "worl."));
-    SpanQuery wrappedquery = new SpanOrQuery(new SpanMultiTermQueryWrapper<RegexpQuery>(regex));
+    SpanQuery wrappedquery = new SpanOrQuery(new SpanMultiTermQueryWrapper<>(regex));
 
     MemoryIndex mindex = new MemoryIndex(random().nextBoolean(),  random().nextInt(50) * 1024 * 1024);
     mindex.addField("field", new MockAnalyzer(random()).tokenStream("field", "hello there"));
@@ -434,6 +430,7 @@ public class MemoryIndexTest extends BaseTokenStreamTestCase {
     for (int i = 0; i < numDocs; i++) {
       Directory dir = newDirectory();
       MockAnalyzer mockAnalyzer = new MockAnalyzer(random());
+      mockAnalyzer.setMaxTokenLength(TestUtil.nextInt(random(), 1, IndexWriter.MAX_TERM_LENGTH));
       IndexWriter writer = new IndexWriter(dir, newIndexWriterConfig(random(), TEST_VERSION_CURRENT, mockAnalyzer));
       Document nextDoc = lineFileDocs.nextDoc();
       Document doc = new Document();
@@ -469,4 +466,80 @@ public class MemoryIndexTest extends BaseTokenStreamTestCase {
     TopDocs docs = searcher.search(new TermQuery(new Term("foo", "")), 10);
     assertEquals(1, docs.totalHits);
   }
+
+  public void testDuelMemoryIndexCoreDirectoryWithArrayField() throws Exception {
+
+    final String field_name = "text";
+    MockAnalyzer mockAnalyzer = new MockAnalyzer(random());
+    if (random().nextBoolean()) {
+      mockAnalyzer.setOffsetGap(random().nextInt(100));
+    }
+    //index into a random directory
+    FieldType type = new FieldType(TextField.TYPE_STORED);
+    type.setStoreTermVectorOffsets(true);
+    type.setStoreTermVectorPayloads(false);
+    type.setStoreTermVectorPositions(true);
+    type.setStoreTermVectors(true);
+    type.freeze();
+
+    Document doc = new Document();
+    doc.add(new Field(field_name, "la la", type));
+    doc.add(new Field(field_name, "foo bar foo bar foo", type));
+
+    Directory dir = newDirectory();
+    IndexWriter writer = new IndexWriter(dir, newIndexWriterConfig(random(), TEST_VERSION_CURRENT, mockAnalyzer));
+    writer.updateDocument(new Term("id", "1"), doc);
+    writer.commit();
+    writer.close();
+    DirectoryReader reader = DirectoryReader.open(dir);
+
+    //Index document in Memory index
+    MemoryIndex memIndex = new MemoryIndex(true);
+    memIndex.addField(field_name, "la la", mockAnalyzer);
+    memIndex.addField(field_name, "foo bar foo bar foo", mockAnalyzer);
+
+    //compare term vectors
+    Terms ramTv = reader.getTermVector(0, field_name);
+    IndexReader memIndexReader = memIndex.createSearcher().getIndexReader();
+    Terms memTv = memIndexReader.getTermVector(0, field_name);
+
+    compareTermVectors(ramTv, memTv, field_name);
+    memIndexReader.close();
+    reader.close();
+    dir.close();
+
+  }
+
+  protected void compareTermVectors(Terms terms, Terms memTerms, String field_name) throws IOException {
+
+    TermsEnum termEnum = terms.iterator(null);
+    TermsEnum memTermEnum = memTerms.iterator(null);
+
+    while (termEnum.next() != null) {
+      assertNotNull(memTermEnum.next());
+      assertThat(termEnum.totalTermFreq(), equalTo(memTermEnum.totalTermFreq()));
+
+      DocsAndPositionsEnum docsPosEnum = termEnum.docsAndPositions(null, null, 0);
+      DocsAndPositionsEnum memDocsPosEnum = memTermEnum.docsAndPositions(null, null, 0);
+      String currentTerm = termEnum.term().utf8ToString();
+
+      assertThat("Token mismatch for field: " + field_name, currentTerm, equalTo(memTermEnum.term().utf8ToString()));
+
+      docsPosEnum.nextDoc();
+      memDocsPosEnum.nextDoc();
+
+      int freq = docsPosEnum.freq();
+      assertThat(freq, equalTo(memDocsPosEnum.freq()));
+      for (int i = 0; i < freq; i++) {
+        String failDesc = " (field:" + field_name + " term:" + currentTerm + ")";
+        int memPos = memDocsPosEnum.nextPosition();
+        int pos = docsPosEnum.nextPosition();
+        assertThat("Position test failed" + failDesc, memPos, equalTo(pos));
+        assertThat("Start offset test failed" + failDesc, memDocsPosEnum.startOffset(), equalTo(docsPosEnum.startOffset()));
+        assertThat("End offset test failed" + failDesc, memDocsPosEnum.endOffset(), equalTo(docsPosEnum.endOffset()));
+        assertThat("Missing payload test failed" + failDesc, docsPosEnum.getPayload(), equalTo(null));
+      }
+    }
+    assertNull("Still some tokens not processed", memTermEnum.next());
+  }
 }
