@@ -19,9 +19,11 @@ package org.apache.lucene.spatial.prefix;
 
 import com.carrotsearch.randomizedtesting.annotations.Repeat;
 import com.spatial4j.core.context.SpatialContext;
+import com.spatial4j.core.context.SpatialContextFactory;
 import com.spatial4j.core.shape.Point;
 import com.spatial4j.core.shape.Rectangle;
 import com.spatial4j.core.shape.Shape;
+import com.spatial4j.core.shape.ShapeCollection;
 import com.spatial4j.core.shape.SpatialRelation;
 import com.spatial4j.core.shape.impl.RectangleImpl;
 import org.apache.lucene.document.Document;
@@ -31,6 +33,7 @@ import org.apache.lucene.document.StringField;
 import org.apache.lucene.search.Query;
 import org.apache.lucene.spatial.StrategyTestCase;
 import org.apache.lucene.spatial.prefix.tree.Cell;
+import org.apache.lucene.spatial.prefix.tree.GeohashPrefixTree;
 import org.apache.lucene.spatial.prefix.tree.QuadPrefixTree;
 import org.apache.lucene.spatial.prefix.tree.SpatialPrefixTree;
 import org.apache.lucene.spatial.query.SpatialArgs;
@@ -40,6 +43,7 @@ import org.junit.Test;
 
 import java.io.IOException;
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.Iterator;
@@ -49,6 +53,7 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 
+import static com.carrotsearch.randomizedtesting.RandomizedTest.randomBoolean;
 import static com.carrotsearch.randomizedtesting.RandomizedTest.randomInt;
 import static com.carrotsearch.randomizedtesting.RandomizedTest.randomIntBetween;
 import static com.spatial4j.core.shape.SpatialRelation.CONTAINS;
@@ -58,62 +63,88 @@ import static com.spatial4j.core.shape.SpatialRelation.WITHIN;
 
 public class SpatialOpRecursivePrefixTreeTest extends StrategyTestCase {
 
-  static final int ITERATIONS = 10;//Test Iterations
+  static final int ITERATIONS = 1;//Test Iterations
 
   private SpatialPrefixTree grid;
+  private SpatialContext ctx2D;
 
-  @Before
-  public void setUp() throws Exception {
-    super.setUp();
-    deleteAll();
-  }
+  public void setupGrid(int maxLevels) throws IOException {
+    if (randomBoolean())
+      setupQuadGrid(maxLevels);
+    else
+      setupGeohashGrid(maxLevels);
+    setupCtx2D(ctx);
 
-  public void mySetup(int maxLevels) throws IOException {
-    //non-geospatial makes this test a little easier (in gridSnap), and using boundary values 2^X raises
-    // the prospect of edge conditions we want to test, plus makes for simpler numbers (no decimals).
-    this.ctx = new SpatialContext(false, null, new RectangleImpl(0, 256, -128, 128, null));
-    //A fairly shallow grid, and default 2.5% distErrPct
-    if (maxLevels == -1)
-      maxLevels = randomIntBetween(1, 8);
-    this.grid = new QuadPrefixTree(ctx, maxLevels);
-    this.strategy = new RecursivePrefixTreeStrategy(grid, getClass().getSimpleName());
     //((PrefixTreeStrategy) strategy).setDistErrPct(0);//fully precise to grid
 
     System.out.println("Strategy: " + strategy.toString());
   }
 
+  private void setupCtx2D(SpatialContext ctx) {
+    if (!ctx.isGeo())
+      ctx2D = ctx;
+    //A non-geo version of ctx.
+    SpatialContextFactory ctxFactory = new SpatialContextFactory();
+    ctxFactory.geo = false;
+    ctxFactory.worldBounds = ctx.getWorldBounds();
+    ctx2D = ctxFactory.newSpatialContext();
+  }
+
+  private void setupQuadGrid(int maxLevels) {
+    //non-geospatial makes this test a little easier (in gridSnap), and using boundary values 2^X raises
+    // the prospect of edge conditions we want to test, plus makes for simpler numbers (no decimals).
+    SpatialContextFactory factory = new SpatialContextFactory();
+    factory.geo = false;
+    factory.worldBounds = new RectangleImpl(0, 256, -128, 128, null);
+    this.ctx = factory.newSpatialContext();
+    //A fairly shallow grid, and default 2.5% distErrPct
+    if (maxLevels == -1)
+      maxLevels = randomIntBetween(1, 8);//max 64k cells (4^8), also 256*256
+    this.grid = new QuadPrefixTree(ctx, maxLevels);
+    this.strategy = new RecursivePrefixTreeStrategy(grid, getClass().getSimpleName());
+  }
+
+  public void setupGeohashGrid(int maxLevels) {
+    this.ctx = SpatialContext.GEO;
+    //A fairly shallow grid, and default 2.5% distErrPct
+    if (maxLevels == -1)
+      maxLevels = randomIntBetween(1, 3);//max 16k cells (32^3)
+    this.grid = new GeohashPrefixTree(ctx, maxLevels);
+    this.strategy = new RecursivePrefixTreeStrategy(grid, getClass().getSimpleName());
+  }
+
   @Test
   @Repeat(iterations = ITERATIONS)
   public void testIntersects() throws IOException {
-    mySetup(-1);
+    setupGrid(-1);
     doTest(SpatialOperation.Intersects);
   }
 
   @Test
   @Repeat(iterations = ITERATIONS)
   public void testWithin() throws IOException {
-    mySetup(-1);
+    setupGrid(-1);
     doTest(SpatialOperation.IsWithin);
   }
 
   @Test
   @Repeat(iterations = ITERATIONS)
   public void testContains() throws IOException {
-    mySetup(-1);
+    setupGrid(-1);
     doTest(SpatialOperation.Contains);
   }
 
   @Test
   @Repeat(iterations = ITERATIONS)
   public void testDisjoint() throws IOException {
-    mySetup(-1);
+    setupGrid(-1);
     doTest(SpatialOperation.IsDisjointTo);
   }
 
   /** See LUCENE-5062, {@link ContainsPrefixTreeFilter#multiOverlappingIndexedShapes}. */
   @Test
   public void testContainsPairOverlap() throws IOException {
-    mySetup(3);
+    setupQuadGrid(3);
     adoc("0", new ShapePair(ctx.makeRectangle(0, 33, -128, 128), ctx.makeRectangle(33, 128, -128, 128), true));
     commit();
     Query query = strategy.makeQuery(new SpatialArgs(SpatialOperation.Contains,
@@ -124,7 +155,7 @@ public class SpatialOpRecursivePrefixTreeTest extends StrategyTestCase {
 
   @Test
   public void testWithinDisjointParts() throws IOException {
-    mySetup(7);
+    setupQuadGrid(7);
     //one shape comprised of two parts, quite separated apart
     adoc("0", new ShapePair(ctx.makeRectangle(0, 10, -120, -100), ctx.makeRectangle(220, 240, 110, 125), false));
     commit();
@@ -138,7 +169,7 @@ public class SpatialOpRecursivePrefixTreeTest extends StrategyTestCase {
 
   @Test /** LUCENE-4916 */
   public void testWithinLeafApproxRule() throws IOException {
-    mySetup(2);//4x4 grid
+    setupQuadGrid(2);//4x4 grid
     //indexed shape will simplify to entire right half (2 top cells)
     adoc("0", ctx.makeRectangle(192, 204, -128, 128));
     commit();
@@ -159,6 +190,16 @@ public class SpatialOpRecursivePrefixTreeTest extends StrategyTestCase {
     ), 1).numFound==1);//match
   }
 
+  @Test
+  public void testShapePair() {
+    ctx = SpatialContext.GEO;
+    setupCtx2D(ctx);
+
+    Shape leftShape = new ShapePair(ctx.makeRectangle(-74, -56, -8, 1), ctx.makeRectangle(-180, 134, -90, 90), true);
+    Shape queryShape = ctx.makeRectangle(-180, 180, -90, 90);
+    assertEquals(SpatialRelation.WITHIN, leftShape.relate(queryShape));
+  }
+
   //Override so we can index parts of a pair separately, resulting in the detailLevel
   // being independent for each shape vs the whole thing
   @Override
@@ -168,7 +209,7 @@ public class SpatialOpRecursivePrefixTreeTest extends StrategyTestCase {
     if (shape != null) {
       Collection<Shape> shapes;
       if (shape instanceof ShapePair) {
-        shapes = new ArrayList<Shape>(2);
+        shapes = new ArrayList<>(2);
         shapes.add(((ShapePair)shape).shape1);
         shapes.add(((ShapePair)shape).shape2);
       } else {
@@ -179,8 +220,8 @@ public class SpatialOpRecursivePrefixTreeTest extends StrategyTestCase {
           doc.add(f);
         }
       }
-      if (storeShape)
-        doc.add(new StoredField(strategy.getFieldName(), ctx.toString(shape)));
+      if (storeShape)//just for diagnostics
+        doc.add(new StoredField(strategy.getFieldName(), shape.toString()));
     }
     return doc;
   }
@@ -195,30 +236,29 @@ public class SpatialOpRecursivePrefixTreeTest extends StrategyTestCase {
 
     final boolean biasContains = (operation == SpatialOperation.Contains);
 
-    Map<String, Shape> indexedShapes = new LinkedHashMap<String, Shape>();
-    Map<String, Shape> indexedShapesGS = new LinkedHashMap<String, Shape>();
+    //Main index loop:
+    Map<String, Shape> indexedShapes = new LinkedHashMap<>();
+    Map<String, Shape> indexedShapesGS = new LinkedHashMap<>();//grid snapped
     final int numIndexedShapes = randomIntBetween(1, 6);
+    boolean indexedAtLeastOneShapePair = false;
     for (int i = 0; i < numIndexedShapes; i++) {
       String id = "" + i;
       Shape indexedShape;
-      Shape indexedShapeGS; //(grid-snapped)
       int R = random().nextInt(12);
       if (R == 0) {//1 in 12
-        indexedShape = null; //no shape for this doc
-        indexedShapeGS = null;
-      } else if (R % 3 == 0) {//4-1 in 12
+        indexedShape = null;
+      } else if (R == 1) {//1 in 12
+        indexedShape = randomPoint();//just one point
+      } else if (R <= 4) {//3 in 12
         //comprised of more than one shape
-        Rectangle shape1 = randomRectangle();
-        Rectangle shape2 = randomRectangle();
-        indexedShape = new ShapePair(shape1, shape2, biasContains);
-        indexedShapeGS = new ShapePair(gridSnap(shape1), gridSnap(shape2), biasContains);
+        indexedShape = randomShapePairRect(biasContains);
+        indexedAtLeastOneShapePair = true;
       } else {
-        //just one shape
-        indexedShape = randomRectangle();
-        indexedShapeGS = gridSnap(indexedShape);
+        indexedShape = randomRectangle();//just one rect
       }
+
       indexedShapes.put(id, indexedShape);
-      indexedShapesGS.put(id, indexedShapeGS);
+      indexedShapesGS.put(id, gridSnap(indexedShape));
 
       adoc(id, indexedShape);
 
@@ -226,6 +266,7 @@ public class SpatialOpRecursivePrefixTreeTest extends StrategyTestCase {
         commit();//intermediate commit, produces extra segments
 
     }
+    //delete some documents randomly
     Iterator<String> idIter = indexedShapes.keySet().iterator();
     while (idIter.hasNext()) {
       String id = idIter.next();
@@ -238,44 +279,62 @@ public class SpatialOpRecursivePrefixTreeTest extends StrategyTestCase {
 
     commit();
 
+    //Main query loop:
     final int numQueryShapes = atLeast(20);
     for (int i = 0; i < numQueryShapes; i++) {
       int scanLevel = randomInt(grid.getMaxLevels());
       ((RecursivePrefixTreeStrategy) strategy).setPrefixGridScanLevel(scanLevel);
-      final Shape queryShape = randomRectangle();
 
-      final boolean DISJOINT = operation.equals(SpatialOperation.IsDisjointTo);
+      final Shape queryShape;
+      switch (randomInt(10)) {
+        case 0: queryShape = randomPoint(); break;
+// LUCENE-5549
+//TODO debug: -Dtests.method=testWithin -Dtests.multiplier=3 -Dtests.seed=5F5294CE2E075A3E:AAD2F0F79288CA64
+//        case 1:case 2:case 3:
+//          if (!indexedAtLeastOneShapePair) { // avoids ShapePair.relate(ShapePair), which isn't reliable
+//            queryShape = randomShapePairRect(!biasContains);//invert biasContains for query side
+//            break;
+//          }
+        default: queryShape = randomRectangle();
+      }
+      final Shape queryShapeGS = gridSnap(queryShape);
+
+      final boolean opIsDisjoint = operation == SpatialOperation.IsDisjointTo;
 
       //Generate truth via brute force:
-      // We really try to ensure true-positive matches (if the predicate on the raw shapes match
+      // We ensure true-positive matches (if the predicate on the raw shapes match
       //  then the search should find those same matches).
       // approximations, false-positive matches
-      Set <String> expectedIds = new LinkedHashSet<String>();//true-positives
-      Set<String> secondaryIds = new LinkedHashSet<String>();//false-positives (unless disjoint)
+      Set<String> expectedIds = new LinkedHashSet<>();//true-positives
+      Set<String> secondaryIds = new LinkedHashSet<>();//false-positives (unless disjoint)
       for (Map.Entry<String, Shape> entry : indexedShapes.entrySet()) {
+        String id = entry.getKey();
         Shape indexedShapeCompare = entry.getValue();
         if (indexedShapeCompare == null)
           continue;
         Shape queryShapeCompare = queryShape;
-        String id = entry.getKey();
+
         if (operation.evaluate(indexedShapeCompare, queryShapeCompare)) {
           expectedIds.add(id);
-          if (DISJOINT) {
+          if (opIsDisjoint) {
             //if no longer intersect after buffering them, for disjoint, remember this
-            indexedShapeCompare = indexedShapesGS.get(entry.getKey());
-            queryShapeCompare = gridSnap(queryShape);
+            indexedShapeCompare = indexedShapesGS.get(id);
+            queryShapeCompare = queryShapeGS;
             if (!operation.evaluate(indexedShapeCompare, queryShapeCompare))
               secondaryIds.add(id);
           }
-        } else if (!DISJOINT) {
+        } else if (!opIsDisjoint) {
           //buffer either the indexed or query shape (via gridSnap) and try again
-          if (operation.equals(SpatialOperation.Intersects)) {
-            indexedShapeCompare = indexedShapesGS.get(entry.getKey());
-            queryShapeCompare = gridSnap(queryShape);
-          } else if (operation.equals(SpatialOperation.Contains)) {
-            indexedShapeCompare = indexedShapesGS.get(entry.getKey());
-          } else if (operation.equals(SpatialOperation.IsWithin)) {
-            queryShapeCompare = gridSnap(queryShape);
+          if (operation == SpatialOperation.Intersects) {
+            indexedShapeCompare = indexedShapesGS.get(id);
+            queryShapeCompare = queryShapeGS;
+            //TODO Unfortunately, grid-snapping both can result in intersections that otherwise
+            // wouldn't happen when the grids are adjacent. Not a big deal but our test is just a
+            // bit more lenient.
+          } else if (operation == SpatialOperation.Contains) {
+            indexedShapeCompare = indexedShapesGS.get(id);
+          } else if (operation == SpatialOperation.IsWithin) {
+            queryShapeCompare = queryShapeGS;
           }
           if (operation.evaluate(indexedShapeCompare, queryShapeCompare))
             secondaryIds.add(id);
@@ -284,17 +343,19 @@ public class SpatialOpRecursivePrefixTreeTest extends StrategyTestCase {
 
       //Search and verify results
       SpatialArgs args = new SpatialArgs(operation, queryShape);
+      if (queryShape instanceof ShapePair)
+        args.setDistErrPct(0.0);//a hack; we want to be more detailed than gridSnap(queryShape)
       Query query = strategy.makeQuery(args);
       SearchResults got = executeQuery(query, 100);
-      Set<String> remainingExpectedIds = new LinkedHashSet<String>(expectedIds);
+      Set<String> remainingExpectedIds = new LinkedHashSet<>(expectedIds);
       for (SearchResult result : got.results) {
         String id = result.getId();
         boolean removed = remainingExpectedIds.remove(id);
-        if (!removed && (!DISJOINT && !secondaryIds.contains(id))) {
+        if (!removed && (!opIsDisjoint && !secondaryIds.contains(id))) {
           fail("Shouldn't match", id, indexedShapes, indexedShapesGS, queryShape);
         }
       }
-      if (DISJOINT)
+      if (opIsDisjoint)
         remainingExpectedIds.removeAll(secondaryIds);
       if (!remainingExpectedIds.isEmpty()) {
         String id = remainingExpectedIds.iterator().next();
@@ -303,11 +364,16 @@ public class SpatialOpRecursivePrefixTreeTest extends StrategyTestCase {
     }
   }
 
-  private void fail(String label, String id, Map<String, Shape> indexedShapes, Map<String, Shape> indexedShapesGS, Shape queryShape) {
-    System.err.println("Ig:" + indexedShapesGS.get(id) + " Qg:" + gridSnap(queryShape));
-    fail(label + " I #" + id + ":" + indexedShapes.get(id) + " Q:" + queryShape);
+  private Shape randomShapePairRect(boolean biasContains) {
+    Rectangle shape1 = randomRectangle();
+    Rectangle shape2 = randomRectangle();
+    return new ShapePair(shape1, shape2, biasContains);
   }
 
+  private void fail(String label, String id, Map<String, Shape> indexedShapes, Map<String, Shape> indexedShapesGS, Shape queryShape) {
+    System.err.println("Ig:" + indexedShapesGS.get(id) + " Qg:" + gridSnap(queryShape));
+    fail(label + " I#" + id + ":" + indexedShapes.get(id) + " Q:" + queryShape);
+  }
 
 //  private Rectangle inset(Rectangle r) {
 //    //typically inset by 1 (whole numbers are easy to read)
@@ -315,7 +381,16 @@ public class SpatialOpRecursivePrefixTreeTest extends StrategyTestCase {
 //    return ctx.makeRectangle(r.getMinX() + d, r.getMaxX() - d, r.getMinY() + d, r.getMaxY() - d);
 //  }
 
-  protected Rectangle gridSnap(Shape snapMe) {
+  protected Shape gridSnap(Shape snapMe) {
+    if (snapMe == null)
+      return null;
+    if (snapMe instanceof ShapePair) {
+      ShapePair me = (ShapePair) snapMe;
+      return new ShapePair(gridSnap(me.shape1), gridSnap(me.shape2), me.biasContainsThenWithin);
+    }
+    if (snapMe instanceof Point) {
+      snapMe = snapMe.getBoundingBox();
+    }
     //The next 4 lines mimic PrefixTreeStrategy.createIndexableFields()
     double distErrPct = ((PrefixTreeStrategy) strategy).getDistErrPct();
     double distErr = SpatialArgs.calcDistanceFromErrPct(snapMe, distErrPct, ctx);
@@ -323,55 +398,86 @@ public class SpatialOpRecursivePrefixTreeTest extends StrategyTestCase {
     List<Cell> cells = grid.getCells(snapMe, detailLevel, false, true);
 
     //calc bounding box of cells.
-    double minX = Double.POSITIVE_INFINITY, maxX = Double.NEGATIVE_INFINITY;
-    double minY = Double.POSITIVE_INFINITY, maxY = Double.NEGATIVE_INFINITY;
+    List<Shape> cellShapes = new ArrayList<>(cells.size());
     for (Cell cell : cells) {
-      assert cell.getLevel() <= detailLevel;
-      Rectangle cellR = cell.getShape().getBoundingBox();
-
-      minX = Math.min(minX, cellR.getMinX());
-      maxX = Math.max(maxX, cellR.getMaxX());
-      minY = Math.min(minY, cellR.getMinY());
-      maxY = Math.max(maxY, cellR.getMaxY());
+      cellShapes.add(cell.getShape());
     }
-    return ctx.makeRectangle(minX, maxX, minY, maxY);
+    return new ShapeCollection<>(cellShapes, ctx).getBoundingBox();
   }
 
   /**
-   * An aggregate of 2 shapes. Only implements what's necessary for the test
-   * here. TODO replace with Spatial4j trunk ShapeCollection.
+   * An aggregate of 2 shapes. Unfortunately we can't simply use a ShapeCollection because:
+   * (a) ambiguity between CONTAINS & WITHIN for equal shapes, and
+   * (b) adjacent pairs could as a whole contain the input shape.
+   * The tests here are sensitive to these matters, although in practice ShapeCollection
+   * is fine.
    */
-  private class ShapePair implements Shape {
+  private class ShapePair extends ShapeCollection<Shape> {
 
-    final Rectangle shape1, shape2;
-    final boolean biasContainsThenWithin;//a hack
+    final Shape shape1, shape2;
+    final Shape shape1_2D, shape2_2D;//not geo (bit of a hack)
+    final boolean biasContainsThenWithin;
 
-    public ShapePair(Rectangle shape1, Rectangle shape2, boolean containsThenWithin) {
+    public ShapePair(Shape shape1, Shape shape2, boolean containsThenWithin) {
+      super(Arrays.asList(shape1, shape2), ctx);
       this.shape1 = shape1;
       this.shape2 = shape2;
+      this.shape1_2D = toNonGeo(shape1);
+      this.shape2_2D = toNonGeo(shape2);
       biasContainsThenWithin = containsThenWithin;
     }
 
+    private Shape toNonGeo(Shape shape) {
+      if (!ctx.isGeo())
+        return shape;//already non-geo
+      if (shape instanceof Rectangle) {
+        Rectangle rect = (Rectangle) shape;
+        if (rect.getCrossesDateLine()) {
+          return new ShapePair(
+              ctx2D.makeRectangle(rect.getMinX(), 180, rect.getMinY(), rect.getMaxY()),
+              ctx2D.makeRectangle(-180, rect.getMaxX(), rect.getMinY(), rect.getMaxY()),
+              biasContainsThenWithin);
+        } else {
+          return ctx2D.makeRectangle(rect.getMinX(), rect.getMaxX(), rect.getMinY(), rect.getMaxY());
+        }
+      }
+      //no need to do others; this addresses the -180/+180 ambiguity corner test problem
+      return shape;
+    }
+
     @Override
     public SpatialRelation relate(Shape other) {
       SpatialRelation r = relateApprox(other);
-      if (r != INTERSECTS && !(r == WITHIN && biasContainsThenWithin))
+      if (r == DISJOINT)
         return r;
+      if (r == CONTAINS)
+        return r;
+      if (r == WITHIN && !biasContainsThenWithin)
+        return r;
+
       //See if the correct answer is actually Contains, when the indexed shapes are adjacent,
       // creating a larger shape that contains the input shape.
-      Rectangle oRect = (Rectangle)other;
       boolean pairTouches = shape1.relate(shape2).intersects();
       if (!pairTouches)
         return r;
       //test all 4 corners
-      if (relate(ctx.makePoint(oRect.getMinX(), oRect.getMinY())) == CONTAINS
-          && relate(ctx.makePoint(oRect.getMinX(), oRect.getMaxY())) == CONTAINS
-          && relate(ctx.makePoint(oRect.getMaxX(), oRect.getMinY())) == CONTAINS
-          && relate(ctx.makePoint(oRect.getMaxX(), oRect.getMaxY())) == CONTAINS)
+      // Note: awkwardly, we use a non-geo context for this because in geo, -180 & +180 are the same place, which means
+      //  that "other" might wrap the world horizontally and yet all it's corners could be in shape1 (or shape2) even
+      //  though shape1 is only adjacent to the dateline. I couldn't think of a better way to handle this.
+      Rectangle oRect = (Rectangle)other;
+      if (cornerContainsNonGeo(oRect.getMinX(), oRect.getMinY())
+          && cornerContainsNonGeo(oRect.getMinX(), oRect.getMaxY())
+          && cornerContainsNonGeo(oRect.getMaxX(), oRect.getMinY())
+          && cornerContainsNonGeo(oRect.getMaxX(), oRect.getMaxY()) )
         return CONTAINS;
       return r;
     }
 
+    private boolean cornerContainsNonGeo(double x, double y) {
+      Shape pt = ctx2D.makePoint(x, y);
+      return shape1_2D.relate(pt).intersects() || shape2_2D.relate(pt).intersects();
+    }
+
     private SpatialRelation relateApprox(Shape other) {
       if (biasContainsThenWithin) {
         if (shape1.relate(other) == CONTAINS || shape1.equals(other)
@@ -387,30 +493,10 @@ public class SpatialOpRecursivePrefixTreeTest extends StrategyTestCase {
       }
 
       if (shape1.relate(other).intersects() || shape2.relate(other).intersects())
-        return INTERSECTS;//might actually be 'CONTAINS' if these 2 are adjacent
+        return INTERSECTS;//might actually be 'CONTAINS' if the pair are adjacent but we handle that later
       return DISJOINT;
     }
 
-    @Override
-    public Rectangle getBoundingBox() {
-      return ctx.getWorldBounds();//good enough
-    }
-
-    @Override
-    public boolean hasArea() {
-      return true;
-    }
-
-    @Override
-    public double getArea(SpatialContext ctx) {
-      throw new UnsupportedOperationException("TODO unimplemented");//TODO
-    }
-
-    @Override
-    public Point getCenter() {
-      throw new UnsupportedOperationException("TODO unimplemented");//TODO
-    }
-
     @Override
     public String toString() {
       return "ShapePair(" + shape1 + " , " + shape2 + ")";
